/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005-2007 by Tobias Nadler                                    */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/
/* here are some common constants and defs */
/*******************************************/

#ifndef DEFINITIONS_H
#define DEFINITIONS_H

/**
@defgroup noclass Global functions and definitions
@version 0.8
@author Tobias Nadler (nadlerto at gmx dot de)

There are also some global functions that do not belong to any class. These and
global definitiones are described here.
*/

/**
@ingroup noclass
@file definitions.h

Often used definitions.
*/

/**
 * Similar to the SDL_Rect struct, but contains float values instead of short.
 */
struct FloatRect
{
	float x, y;
	float w, h;
};

/**
 * This expression should be used whenever the color-value is in the
 * ARGB-byte-order (alpha is most significant byte, blue is least significant
 * byte). If a color-value has a special surface-format, that might be
 * different to ARGB, then you should not use this, but Uint32 or unsigned
 * long. In Netrinjo an alpha value of 255 indicates full transparency
 * (invisible), zero indicates full opacity (no transparency).
 */
#define ARGB_COLOR		Uint32

/// used to indicate no color or full transparency
#define CLR_INVALID		0xFFFFFFFF
/// used to indicate a standard color
#define CLR_STANDARD	(CLR_INVALID - 1)

/// when used as time, indicates never ending duration or a never occuring time
#define INFINITE		0xFFFFFFFF
/// the maximum value, a variable of the Uint_32 type can have
#define MAX_UINT32		0xFFFFFFFF
/// the maximum value, a variable of the Sint_32 type can have
#define MAX_SINT32		0x7FFFFFFF
/// the minimum value, a variable of the Sint_32 type can have
#define MIN_SINT32		0x80000000
/// the maximum value, a variable of the Uint_16 type can have
#define MAX_UINT16		0xFFFF
/// the maximum value, a variable of the Sint_16 type can have
#define MAX_SINT16		0x7FFF
/// the minimum value, a variable of the Sint_16 type can have
#define MIN_SINT16		0x8000
/// the maximum value, a variable of the Uint_8 type can have
#define MAX_BYTE		0xFF
/// the maximum value, a variable of the Sint_8 type can have
#define MAX_CHAR		0x7F
/// the minimum value, a variable of the Sint_8 type can have
#define MIN_CHAR		0x80

/// Used with messageboxes to show an OK button
#define SMB_OK			0x01
/// Used with messageboxes to show a YES button
#define SMB_YES			0x02
/// Used with messageboxes to show a CANCEL button
#define SMB_CANCEL		0x04
/// Used with messageboxes to show a NO button
#define SMB_NO			0x08
/// Used with messageboxes to show an OK and a CANCEL button
#define SMB_OKCANCEL	0x05
/// Used with messageboxes to show a YES and a NO button
#define SMB_YESNO		0x0A
/// Used with messageboxes to show a YES, a NO and a CANCEL button
#define SMB_YESNOCANCEL	0x0E

/// @returns the bigger value of a and b
#ifndef MAX
#define MAX(a,b)			(((a) > (b)) ? (a) : (b))
#endif
/// @returns the smaller value of a and b
#ifndef MIN
#define MIN(a,b)            (((a) < (b)) ? (a) : (b))
#endif

/// @returns the sqare of X
#define sqr(X)	(X*X)
/// @returns true, if i is between min and max, false if not (equal to obsolete ISBETW)
#define BETWEEN(i,min,max)	((i >= min)&&(i <= max))
/// @returns true, if x equals i1 or i2 (equal to obsolete ISOR2)
#define CMP_WITH_2(x,i1,i2)	((x == i1)||(x == i2))

//#define EULER	2.7182818284590//4523536028747135266 //Euler'sche Zahl

#endif

