/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUICURSOR_H
#define GUICURSOR_H

#include "SDL/SDL.h"
#include "guibackground.h"
#include "tinyxml.h"

/**
\ingroup gui
\file guicursor.h
\brief Defines the class GuiCursor.
*/

/**
\ingroup gui
\class GuiCursor
\brief A class for showing a simple mouse cursor

This class is used to show a mouse cursor on the display.
It is hidden after creation, but automatically becomes visible when its
onMouseMove() is called.
*/
class DECLSPEC GuiCursor : public GuiPictureBg
{
public:
	/**
	Constructor. The hot point is set here. The cursor is hidden by default!
	\param p the parent element
	\param bmp the picture for the cursor's look
	\param x the x coordinate of the hot point
	\param y the y coordinate of the hot point
	*/
	DECLSPEC GuiCursor( const SDL_Rect &r, GuiElement *p, const char *name, SdlImage *bmp, Uint16 x, Uint16 y );
	DECLSPEC GuiCursor( TiXmlElement *el, GuiElement *p );
	virtual DECLSPEC ~GuiCursor();

	virtual const char *getClassName()	{ return "GuiCursor"; }
	/**
	Sets the hot point of the cursor (the position, where the
	cursor is, relative to the position of the cursor's
	top-left corner).
	\param x the x coordinate of the hot point
	\param y the y coordinate of the hot point
	*/
	void setHotPoint( Uint16 x, Uint16 y )	{ xHot = x; yHot = y; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiCursor((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}

protected:
	/**
	This event is called, when the mouse cursor was moved.
	\param x the new, absolute x coordinate of the cursor
	\param y the new, absolute y coordinate of the cursor
	\return currently the result is not used; may be zero, if no error...
	*/
	virtual DECLSPEC Uint32 onMouseMove( Uint32 x, Uint32 y );
	virtual DECLSPEC void onInit();

private:
	// the coordinate of the hot point (relative to the cursor's top-left position)
	int xHot, yHot;
};

#endif
