/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIFILEELEMENT_H
#define GUIFILEELEMENT_H

#include "guielement.h"

/**
\ingroup gui
\file guifileelement.h
\brief Defines the class GuiFileElement.
*/

/**
 * \class GuiFileElement
 * \brief A GUI class that creates subelements as specified in a XML-file.
 * 
 * This class can be used to distribute the xml-data to more files or to
 * create elements during run-time using a xml-file.
 * 
 * XML-files that are specified to be parsed, should have a root element named
 * as the classname (here "GuiFileElement"; if you derive a class from this,
 * then the value from getClassname() should be used).
 */
class DECLSPEC GuiFileElement : public GuiElement
{
public:
	/**
	\param filename the XML-file to use for initialization, NULL if no child elements should be created here
	*/
	DECLSPEC GuiFileElement( SDL_Rect &r, GuiElement *el, const char *name, const char *filename = NULL );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the attribute Filename can be specified for initialization
	\param p parent element
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiFileElement(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFileElement();
	/**
	Destroys all child elements and creates new child elements as defined in the specified XML-file.
	The onInit() method of the newly created elements is called after their creation.
	\param fn the XML-file's filename
	\returns true on success (if XML-file could be loaded and parsed), false on failure
	*/
	DECLSPEC bool initByFile(const char *fn);

	virtual const char *getClassName()	{ return "GuiFileElement"; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFileElement((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC void onInit();
	
private:
	// returns the number of children (in 1st generation) added to this element
	int create();
	// the XML data that was used to create the currently existing children
	TiXmlDocument xmlChildrenDoc;
};

#endif
