/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIFILELIST_H
#define GUIFILELIST_H

#include "guilist.h"
#include <vector>
#include <string>
#ifndef MAX_PATH
#define MAX_PATH	256
#endif

/**
\ingroup gui
\file guifilelist.h
\brief Defines the class GuiFileList
*/

/**
\ingroup gui
\class GuiFileList
\brief A class for displaying a listbox containing filenames.
*/
class DECLSPEC GuiFileList : public GuiList
{
public:
	struct FIND_DATA
	{
		Uint32 ulType;
		char chFileName[MAX_PATH];
	};
	DECLSPEC GuiFileList(const SDL_Rect &r, GuiElement * el, const char *name);
	/**
	 * Constructor. Uses XML data.
	 * \param el the XML data for initialisation; possible attributes are:
	 * Path (string), Pattern (string), ShowFiles, ShowDirecories, ShowOthers,
	 * HideExtensions (last four: bool)
	 * \param p parent element (or NULL, to take GuiManager::the as parent)
	 * \see GuiElement(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiFileList(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiFileList();

	virtual const char *getClassName()	{ return "GuiFileList"; }
	
	/**
	 * \param i the item-index of the filename you want
	 * \returns the filename at the specified item-index, this result has to be
	 * freed after usage! It returns NULL if no path is specified or the index
	 * is out of range.
	 * \note the filename consists of the path as specified in setPath() and
	 * the filename as shown in the list.
	 */
	DECLSPEC char * getFilename(unsigned int i) const;
	/**
	 * \returns the selected filename or NULL if none is selected or no path
	 * specified. If non-NULL the result has to be freed after usage!
	 * \note the filename consists of the path as specified in setPath() and
	 * the filename as shown in the list.
	 */
	DECLSPEC char * getSelFilename() const;
	DECLSPEC FIND_DATA getAttribute(unsigned int i) const;
	DECLSPEC FIND_DATA getSelAttr() const;

	DECLSPEC void setFiletypes( bool files, bool dirs, bool others = false );
	/**
	 * Shows the files in the specified directory.
	 * \param path the path to the directory to display; may be relative to the
	 * user's data directory (as in BasicApp::getUserdataPath())
	 */
	DECLSPEC void setPath(const char * path);
	DECLSPEC void setSeekstr(const char * str);
	DECLSPEC void hideExtension(bool hide = true);

	DECLSPEC void refresh();

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiFileList((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}

protected:
	virtual DECLSPEC void onInit();

private:
	Uint32 showtypes;
	bool hideext;
	char * seekstr;
	// the absolute path to the shown folder
	std::string strPath;
	typedef vector<FIND_DATA> attrlist;
	attrlist attributes;
};

#endif
