/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIHINT_H
#define GUIHINT_H

#include "guitext.h"
#include "frametimer.h"

/**
\ingroup gui
\file guihint.h
\brief Defines the class GuiHint.
*/

/**
\ingroup gui
\class GuiHint
\brief A class to display hints.
*/
class DECLSPEC GuiHint : public GuiFlowWindow
{
public:
	DECLSPEC GuiHint( const SDL_Rect &r, GuiElement *el, const char *name );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiHint(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiHint();
	
	virtual const char *getClassName()	{ return "GuiHint"; }
	
	DECLSPEC void show( int time );
	void setText( const char *t )	{ text->setText(t); }
	void setTextfont( bool l, GuiText::FontSize s, bool std = false )	{ text->chooseFont( s, l ); }
	DECLSPEC void showHint( const char *txt, int time = 0, SDL_Rect *r = 0 );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiHint((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	void onTimer();
	FrameTimer::CbInfo * timerinfo;
	GuiText *text;
	static inline void onTimer( void *sender, BasicObject *aim, void * data )
	{
		( (GuiHint*)aim )->onTimer();
	}
	Uint32 tShow;

	friend class UiManager;
};

#endif
