/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUILIST_H
#define GUILIST_H

#include "guibackground.h"
#include "guiscroller.h"
#include <vector>

/**
\ingroup gui
\file guilist.h
\brief Defines the class GuiList.
*/

using namespace std;

/**
\ingroup gui
\class GuiList
\brief A class for displaying lists
*/
class DECLSPEC GuiList : public GuiPictureBg
{
public:
	DECLSPEC GuiList( const SDL_Rect &r, GuiElement *p, const char *name );
	/**
	 * Constructor. Uses XML data.
	 * \param el the XML data for initialisation, evaluated attributes are:
	 * OnSelect (callback) and Entry<id> (string).
	 * \param p parent element (or NULL, to take GuiManager::the as parent)
	 * \see GuiElement(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiList(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiList();

	virtual const char *getClassName()	{ return "GuiList"; }
	
	/**
	 * Inserts a string into the list.
	 * @param str the string to insert
	 * @param pos the index where to insert it (0 to put it at the beginning),
	 *            -1 to put it at the list's end
	 */
	DECLSPEC int insertStr( const char* str, int pos = -1 );
	DECLSPEC bool clearStr( int pos = -1 );
	DECLSPEC void clearAll();
	DECLSPEC bool changeStr( const char *newstr, int pos );
	DECLSPEC const char *getStr( int pos ) const;
	DECLSPEC const char *getSelStr() const;
	DECLSPEC int getSelId() const;
	DECLSPEC int getCount() const;
	
	void setSelChangeCallback( Callback c )	{ cbSelchanged.func = c; }
	void setCallbackData( BasicObject * d )	{ cbSelchanged.obj = d; }
	void setSelected( int i )	{ selected = i; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiList((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );
private:
	typedef vector<char*> strlist;
	typedef strlist::iterator strlist_iter;

	void arrange();
	strlist_iter iterfrompos(int pos);
	void update();
	void updatefromscroller();

	strlist strings;
	int selected;
	int topitem;

	GuiScroller *scroller;
	Uint32 nMaxScroll;
	SdlFont *font, *fontSel;

	FunctionData cbSelchanged;

	static inline void ontopitemchanged(void*, BasicObject *data, void*)
	{
		GuiList *list = (GuiList*)data;
		list->updatefromscroller();
	}
};

#endif
