/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2005 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUILISTBOX_H
#define GUILISTBOX_H

#include "guilist.h"

/**
\ingroup gui
\file guilistbox.h
\brief Defines the class GuiListBox.
*/

class GuiList;
class GuiText;
class GuiTextInput;
class GuiClickableBtn;

/**
 * \ingroup gui
 * \class GuiListBox
 * \brief A listbox only shows the selected entry - if not focused.
 * 
 * If the user selects the listbox, a GuiList with all selectable entries pops
 * up.
 */
class DECLSPEC GuiListBox : public GuiElement
{
public:
	DECLSPEC GuiListBox( const SDL_Rect &r, GuiElement *p, const char *name, bool editable = false );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiListBox(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiListBox();

	virtual const char *getClassName()	{ return "GuiListBox"; }
	
	DECLSPEC void destroy();
//	void setCallback( Callback c, BasicObject *d )	{ cb.func = c; cb.obj = d; }

	int insertStr( const char* str, int pos = -1 )	{ return gList->insertStr( str, pos ); }
	bool clearStr( int pos = -1 )	{ return gList->clearStr( pos ); }
	void clearAll()	{ gList->clearAll(); }
	bool changeStr( const char *newstr, int pos )	{ return gList->changeStr( newstr, pos ); }
	const char *getStr( int pos ) const	{ return gList->getStr( pos ); }
	const char *getSelStr() const	{ return gList->getSelStr(); }
	int getSelId() const	{ gList->getSelId(); }
	int getCount() const	{ gList->getCount(); }
	
	void setSelected(int i)	{ gList->setSelected( i );}
	
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiListBox((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC void onInit();
	
private:
	void onButton();
	void onList();

	GuiList *gList;
	GuiText *gText;
	GuiTextInput *gtInput;
	GuiClickableBtn *gcButton;

//	FunctionData cb;

	static void onButton( void *sender, BasicObject *data, void* )
	{
		((GuiListBox*)data)->onButton();
	}
	static void onList( void *sender, BasicObject *data, void* )
	{
		((GuiListBox*)data)->onList();
	}
};

#endif
