/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIMESSAGEBOX_H
#define GUIMESSAGEBOX_H

#include "guielement.h"
#include "guibackground.h"
#include "guitext.h"
#include "guidynamicbutton.h"

/**
\ingroup gui
\file guimessagebox.h
\brief Defines the class GuiMessageBox.
*/

/**
\ingroup gui
\class GuiMessageBox
\brief A class for message boxes.
*/
class DECLSPEC GuiMessageBox : public GuiPictureBg
{
public:
	DECLSPEC GuiMessageBox( const SDL_Rect &r, GuiElement * el, const char *name, const char * t = NULL, int buttons = -1 );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiMessageBox(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiMessageBox();

	virtual const char *getClassName()	{ return "GuiMessageBox"; }

	void setCallback( Callback c, BasicObject * d )	{ cb.func = c; cb.obj = d; }
	DECLSPEC void showMessage( const char *m, long buttons = -1);
	DECLSPEC void setButtons( long i );
	DECLSPEC int getAnswer( bool hide = true );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiMessageBox((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();

private:
	void free();
	void onButton( GuiClickableBtn *sender );
	void arrange();
	int index;
	int nButtons;
	GuiText *text;
	GuiBorder *border;
	GuiClickableBtn *gcbOk, *gcbYes, *gcbCancel, *gcbNo;
	FunctionData cb;
	static inline void onButtonClick( void *sender, BasicObject* data, void* )
	{
		((GuiMessageBox*)data)->onButton( (GuiClickableBtn *)sender );
	}
};

#endif
