/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIPROGRESSBAR_H
#define GUIPROGRESSBAR_H

#include "guibackground.h"
#include "guielement.h"
#include "guitext.h"

/**
\ingroup gui
\file guiprogressbar.h
\brief Defines the class GuiProgressBar.
*/

/**
\ingroup gui
\class GuiProgressBar
\brief A class to display the progress of some calculation.

\test This class is not tested yet, but the static methods are.
*/
class DECLSPEC GuiProgressBar : public GuiPictureBg
{
public:
	/**
	 * Shows the previously initialized progressbar.
	 * \param p the progress (between 0 and 1)
	 * \param barcolor the color of the progressbar
	 * \param bordercolor the color of the border, shown around the bar
	 * \param emptycolor the color of the bar for the missing progress
	 */
	static DECLSPEC void showprogresssimple( float p, ARGB_COLOR barcolor = 0x30C030, ARGB_COLOR bordercolor = 0x0060C0, ARGB_COLOR emptycolor = 0xC06000 );
	/**
	 * Uninitializes the progressbar by destroying the bitmap-surface.
	 * \note the surface is also freed, if it has been initialized somewhere
	 * before initsimple().
	 */
	static DECLSPEC void uninitsimple();
	/**
	 * Initializes a simple progressbar. The progressbar is drawn directly onto
	 * the top-level surface and refreshed by showprogresssimple().
	 * So you see it only, if there are no frames drawn by other objects.
	 * Usually these static methods are used in a single method containing a
	 * loop that ends when the progress is at 100%.
	 * \param waitbmp the name of the image's surface to show above the bar; if
	 * there is none with that name, a new one is created, taking this param as
	 * an image's file-path (using BasicApp::makeDataPath()).
	 */
	static DECLSPEC bool initsimple( const char *waitbmp );

	DECLSPEC GuiProgressBar( const SDL_Rect &r, GuiElement *el, const char *name, SdlImage * progbmp = 0, SdlImage *backbmp = 0 );
	/**
	 * Constructor. Uses XML data.
	 * \param el the XML data for initialisation
	 * \param p parent element (or NULL, to take GuiManager::the as parent)
	 * \see GuiElement(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiProgressBar(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiProgressBar();
	
	virtual const char *getClassName()	{ return "GuiProgressBar"; }
	
	DECLSPEC int operator ++();
	DECLSPEC int operator += ( int i );
	void setFramebmp( SdlImageContainer * r )	{ brd->setBmps( r ); }
	DECLSPEC void doFrame( SdlSurface *dst );
	DECLSPEC void setProgress( int i );
	DECLSPEC void setFont( SdlFont *f );

	DECLSPEC void setValues( int max, int step );
	DECLSPEC void setHorizontal( bool h );

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiProgressBar((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}

protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

	SdlImage *bmp;
	int maxvalue, stepsize;
	bool horizontal;

private:
	void arrange();
	int progress;
	int width;
	GuiBorder * brd;
	GuiText * text;

	bool bThemeBmp, bThemeFont, bThemeVal, bThemeHorz;
	static SdlImage *waitres;
};

#endif
