/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler und Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIROTATOR_H
#define GUIROTATOR_H

#include "guielement.h"

/**
 * \ingroup gui
 * \file guirotator.h
 * \brief Defines the class GuiRotator.
 */

class SdlImage;

/**
 * \ingroup gui
 * \class GuiRotator
 * \brief A class to display rotating pictures.
 * \note the performance is very bad in framebuffer-mode, it is recommended to
 * use this class in OpenGL-mode only.
 */
class DECLSPEC GuiRotator : public GuiElement
{
public:
	DECLSPEC GuiRotator( const SDL_Rect &r, GuiElement *p, const char *name, SdlImage *bmp, double angle = 0 );
	/**
	 * Constructor. Uses XML data.
	 * \param el the XML data for initialisation; evaluated attributes are:
	 * Angle (double [degrees]), Speed (double [degrees per second]),
	 * Zoom (double), Smooth (bool) and RotatingBmp (image resource).
	 * \param p parent element (or NULL, to take GuiManager::the as parent)
	 * \see GuiElement(TiXmlElement*,GuiElement*)
	 */
	DECLSPEC GuiRotator(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiRotator();

	virtual const char *getClassName()	{ return "GuiRotator"; }
	/**
	 * Sets the angle and speed.
	 * \param a the current angle in degrees
	 * \param speed the turning speed in degrees per second
	 */
	DECLSPEC void setAngleSpeed( double a, double speed );
	/// \param a the current angle in degrees
	DECLSPEC void setAngle( double a );
	/// \param speed the turning speed in degrees per second
	DECLSPEC void setSpeed( double speed );
	/// \param s the image to show
	void setBmp( SdlImage *s )	{ sBmp = s; }

	/// the zoom, 1.0 if equal size as resource, 2.0 for double size, 0.5 for
	/// half size, etc.
	double dZoom;
	/// only used in framebuffer-mode; if true, it improves the appearance, but
	/// also requires more processing-time
	bool bSmooth;

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiRotator((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onFrame( SdlSurface *dst );
	virtual DECLSPEC Uint32 onGLFrame();
	virtual DECLSPEC void paintBg( SdlSurface *dst );
	virtual DECLSPEC void paintBgGL();
	virtual DECLSPEC void onInit();

private:
	SdlImage *sBmp;
	double dAngle;
	double dSpeed;// 360 => one round per second
	Uint32 nTime;
};

#endif
