/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler und Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUISCROLLER_H
#define GUISCROLLER_H

#include "guidynamicbutton.h"
#include "guibackground.h"

/**
\ingroup gui
\file guiscroller.h
\brief Defines the classes GuiScroller and GuiScrollBg
*/

/**
\ingroup gui
\class GuiScroller
\brief A class for scrollbars.
*/
class DECLSPEC GuiScroller : public GuiPictureBg  
{
public:
	enum direction {horizontal, vertikal};

	DECLSPEC GuiScroller( const SDL_Rect &r, GuiElement *p, const char *name, bool usetheme = true );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiScroller(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiScroller();

	virtual const char *getClassName()	{ return "GuiScroller"; }
	
	int getScrollPos() const	{ return pos; }
	DECLSPEC int setScrollPos( int p );
	DECLSPEC int setMinMax( int mi, int ma );
	int getMax() const	{ return max; }
	int getMin() const	{ return min; }
	DECLSPEC void setDirection( direction d );

	void setChangeCallback( Callback c )	{ cbChange.func = c; }
	void setCallbackData( BasicObject * d )	{ cbChange.obj = d; }

	int tick;
	
	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiScroller((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onMouseDown( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onMouseUp( SDL_MouseButtonEvent *ev );
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

	GuiClickableBtn *buttonincrement;
	GuiClickableBtn *buttondecrement;
	GuiMoveableBtn *buttonmove;
private:
	void updateitempos();
	void updateitemrange();
	void arrange();
	void increment();
	void decrement();
	void updatefromitem();

	FunctionData cbChange;
	direction dir;
	int pos;
	int min;
	int max;
	bool bThemeBtns;

	static inline void onButton( void* button, BasicObject * data, void* )
	{
		GuiScroller *scroller = (GuiScroller*)data;
		if( (GuiClickableBtn*)button == scroller->buttonincrement )
			scroller->increment();
		else if( (GuiClickableBtn*)button == scroller->buttondecrement )
			scroller->decrement();
		else if( (GuiClickableBtn*)button == scroller->buttonmove )
			scroller->updatefromitem();
	}

	static void setInfo();
	friend class guiinfo;
};

/**
\ingroup gui
\class GuiScrollBg
\brief A class to display a large object in a small area by using scrollbars.
*/
class DECLSPEC GuiScrollBg : public GuiElement
{
public:
	DECLSPEC GuiScrollBg( const SDL_Rect &r, GuiElement *el, const char *name, int minw, int minh, SdlImage * bmp = NULL);
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiScrollBg(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiScrollBg();

	virtual const char *getClassName()	{ return "GuiScrollBg"; }
	
	DECLSPEC void setMinSize( int w, int h );
	GuiFlowWindow *getwnd()				{ return bg; }
	void setCallback( Callback c )		{ cb.func = c; }
	void setCallbackData( BasicObject * d )	{ cb.obj = d; }

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiScrollBg((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();

private:
	void arrange();
	void init(SdlImage * bmp);
	
	int nMinWidth, nMinHeight;
	GuiScroller *scrolly;
	GuiScroller *scrollx;
	GuiFlowWindow *bg;
	GuiPictureBg *gpbCorner;
	FunctionData cb;
	void onScroll( GuiScroller * sender );
	static inline void onScroll( void * sender, BasicObject * data, void* )
	{
		((GuiScrollBg*)data)->onScroll( (GuiScroller *)sender );
	};
};

#endif
