/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef GUIUPDOWN_H
#define GUIUPDOWN_H

#include "guibackground.h"
#include "guitext.h"
#include "guidynamicbutton.h"

/**
\ingroup gui
\file guiupdown.h
\brief Defines the class GuiUpDown.
*/

/**
\ingroup gui
\class GuiUpDown
\brief A class, in which the user can enter a number.
*/
class DECLSPEC GuiUpDown : public GuiPictureBg
{
public:
	DECLSPEC GuiUpDown( const SDL_Rect &r, GuiElement *el, const char *name, int val = 0, bool usetheme = true );
	/**
	Constructor. Uses XML data.
	\param el the XML data for initialisation, the following attributes are
	          read: Value (int), MinValue (int), MaxValue (int), UseTheme (bool),
	          OnChange (event)
	\param p parent element (or NULL, to take GuiManager::the as parent)
	\see GuiElement(TiXmlElement*,GuiElement*)
	*/
	DECLSPEC GuiUpDown(TiXmlElement *el, GuiElement *p);
	virtual DECLSPEC ~GuiUpDown();

	virtual const char *getClassName()	{ return "GuiUpDown"; }
	
	void setCallback( Callback c )		{ cb.func = c; }
	void setCallbackData( BasicObject* d )	{ cb.obj = d; }
	DECLSPEC void setValue( int v );
	DECLSPEC void setMinMax( int min, int max );
	void chooseFont( GuiText::FontSize size, bool light )	{ text->chooseFont( size, light ); }
	DECLSPEC int getValue();

	DECLSPEC int operator ++();
	DECLSPEC int operator --();
	DECLSPEC int operator +=(int i);
	DECLSPEC int operator -=(int i);

	int increment;

	static void newguiobject(void *presult, BasicObject *o, void *xmldata)
	{
		GuiElement *r = new GuiUpDown((TiXmlElement*)xmldata, (GuiElement*)o);
		*(GuiElement**)presult = r;
	}
protected:
	virtual DECLSPEC Uint32 onRelease();
	virtual DECLSPEC Uint32 onResize();
	virtual DECLSPEC void onInit();
	virtual DECLSPEC Uint32 onThemeChanged( UiTheme *t );

private:
	void refresh();
	void arrange();
	void onButton( GuiClickableBtn * button );

	FunctionData cb;
	int value;
	int min, max;
	GuiClickableBtn * up;
	GuiClickableBtn * down;
	GuiTextInput * text;

	bool bThemeBtns;

	static inline void onUdBtn( void * btn, BasicObject *reciever, void * )
	{
		((GuiUpDown*)reciever )->onButton( (GuiClickableBtn*)btn );
	}
	static inline void onText( void*, BasicObject *reciever, void * )
	{
		((GuiUpDown*)reciever )->onRelease();
	}
};

#endif
