/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 - 2007 by Tobias Nadler                                  */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef RECTFUNCTIONS_H
#define RECTFUNCTIONS_H

#ifndef DOXYGEN
#include "SDL/SDL.h"
#include "definitions.h"
#endif

/**
@ingroup noclass
@file rectfunctions.h
@brief Contains methods for calculating with rects.

Required header file:
@code #include <rectfunctions.h> @endcode

*/

/**
Prooves whether a point is inside a rect, or not.
@param r The rect
@param x The x coordinate of the point
@param y The y coordinate of the point
@returns true, if the point is inside the rect, false if not
*/
DECLSPEC bool pointInRect( const SDL_Rect &r, int x, int y );
DECLSPEC bool pointInRect( const FloatRect &r, float x, float y );

/**
Makes a rect invalid by setting x and y to -1 and w and h to 0.
@param r The rect to empty
*/
DECLSPEC void setRectEmpty( SDL_Rect &r );
DECLSPEC void setRectEmpty( FloatRect &r );

/**
Prooves whether a rect is empty or not
@param r The rect to proove
@returns true, if w is zero and x is -1, false otherwise
*/
DECLSPEC bool isRectEmpty( const SDL_Rect &r );
DECLSPEC bool isRectEmpty( const FloatRect &r );

/**
Moves a rect
@param r The rect to move
@param dx The distance to move in x-direction, negative to move to the left
@param dy The distance to move in y-direction, negative to move to the top
*/
DECLSPEC void offsetRect( SDL_Rect &r, int dx, int dy );
DECLSPEC void offsetRect( FloatRect &r, float dx, float dy );

/**
Sets all of the rect's data.
@param r A reference to the rect
@param x the x coordinate
@param y the y coordinate
@param w the width
@param h the height
*/
DECLSPEC void setRect( SDL_Rect &r, int x, int y, int w, int h );
DECLSPEC void setRect( FloatRect &r, float x, float y, float w, float h );

/**
Resizes a rect, the rect's center will not change.
@param r The rect to resize
@param dx The value to change the left and right borders, negative to
make it smaller
@param dy The value to change the top and bottom borders, negative to
make it smaller
*/
DECLSPEC void inflateRect( SDL_Rect &r, int dx, int dy );
DECLSPEC void inflateRect( FloatRect &r, float dx, float dy );

/**
Intersects two rects and assignes the intersected area to r.
@param r One rect, the result will be assigned to this
@param r2 The other rect, is stays unaffected
*/
DECLSPEC void intersectRect( SDL_Rect &r, const SDL_Rect &r2 );
DECLSPEC void intersectRect( FloatRect &r, const FloatRect &r2 );

#endif
