/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILEMUSICLIST_H
#define SDLFILEMUSICLIST_H

#include "sdlmusiclist.h"
#include <fstream>

/**
\ingroup sdl
\file sdlfilemusiclist.h
\brief Defines the class SdlFileMusicList.
*/

using namespace std;

/**
\ingroup sdl
\class SdlFileMusicList
\brief A class to load and manage playlist files

Use this class to load .PLS or .M3U filelists and manage them.

\see SdlMusicList, SdlFileMusicList
*/
class DECLSPEC SdlFileMusicList : public SdlMusicList
{
public:
	/**
	Constructor. Loads a playlist file (format is recognized
	automatically). If the list could not be loaded, the fail-state
	will be set.
	\param fn the filename of the playlist-file to load
	\param name the name of this list-resource
	*/
	DECLSPEC SdlFileMusicList( const char *fn, const char *name = 0 );
	virtual DECLSPEC ~SdlFileMusicList();

private:
	DECLSPEC bool loadM3U( ifstream *file );
	DECLSPEC bool loadPLS( ifstream *file );

	char *chFN;
};

#endif
