/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILESURFACE_H
#define SDLFILESURFACE_H

#include <string>
#include "sdlsurface.h"
#include "sdlmanager.h"
#include "definitions.h"

/**
\ingroup sdl
\file sdlfilesurface.h
\brief Defines the class SdlFileSurface.
*/

/**
\ingroup sdl
\class SdlFileSurface
\brief A class for a graphical surface resource, loaded from a file.

The class supports the initialisation by specifying a file of one
of the following formats:\n
BMP, JPG, PNG, TIF, PCX, TGA, PNM, XPM, XCF and LBM.
*/
class DECLSPEC SdlFileSurface : public SdlSurface
{
public:
	/**
	Constructor. Registers itself in SdlManager with the given name
	or, if name is NULL, with the given filename.
	After the construction, a surface buffer, initialized with the
	data of the given filename, the colorkey and flags is availible
	for usage. If initialisation failed, the fail-state is set.
	\param fn the filename of the image file
	\param name the resource's name
	\param colorkey the color, that is fully transparent when blitting
	\param type the type of the image file as string with 3 letters
	(as listed above); if NULL, the ending of the filename is used to
	detect the type
	\param flags the flags of the surface, for details, have a look
	at the SDL documentation; usually SDL_HWSURFACE is the best value.
	\see SdlManager::getSurfaceByName()
	*/
	DECLSPEC SdlFileSurface( const char *fn, const char *name = 0, ARGB_COLOR colorkey = CLR_INVALID, const char *type = 0, Uint32 flags = SDL_HWSURFACE );
	virtual DECLSPEC ~SdlFileSurface();

protected:
	/**
	Refreshes the surface, by loading the buffer from the computer's RAM
	to the graphical RAM, if SDL_HWSURFACE used.
	This is neccessary, when resoring an application in MS Windows
	\todo test this (I think it does not function yet) and make it working.
	*/
	virtual DECLSPEC void onRefresh();

private:
	std::string strFilename;
	ARGB_COLOR ulColorKey;
	Uint32 ulFlags;
	char chType[4];
};

#endif
