/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFILETEXTURE_H
#define SDLFILETEXTURE_H

#include <string>
#include "sdltexture.h"
#include "sdlmanager.h"
#include "definitions.h"

/**
 * \ingroup sdl
 * \file sdlfiletexture.h
 * \brief Defines the class SdlFileTexture.
 */

/**
 * \ingroup sdl
 * \class SdlFileTexture
 * \brief A class for a graphical surface resource, loaded from a file.
 * 
 * The class supports the initialisation by specifying a file of one
 * of the following formats:\n
 * BMP, JPG, PNG, TIF, PCX, TGA, PNM, XPM, XCF and LBM.
 */
class DECLSPEC SdlFileTexture : public SdlTexture
{
public:
	/**
	 * Constructor. Registers itself in SdlManager with the given name
	 * or, if name is NULL, with the given filename.
	 * After the construction, a texture buffer, initialized with the
	 * data of the given filename, the colorkey and flags is availible
	 * for usage. If initialisation failed, the fail-state is set.
	 * \param fn the filename of the image file
	 * \param name the resource's name
	 * \param colorkey the color, that is fully transparent when blitting
	 * (only usable if the image has no alpha values)
	 * \param type the type of the image file as string with 3 letters
	 * (as listed above); if NULL, the ending of the filename is used to
	 * detect the type
	 * \param mirror true to mirror the image on the x-achsis
	 * \see SdlManager::getImageByName()
	 */
	DECLSPEC SdlFileTexture( const char *fn, const char *name = 0, ARGB_COLOR colorkey = CLR_INVALID, const char *type = 0, bool mirror = false );
	virtual DECLSPEC ~SdlFileTexture();

protected:
	/**
	 * Refreshes the texture, by loading the buffer from the computer's RAM
	 * to the graphic RAM.
	 * This is neccessary, i.e. on MS-Windows-systems when restoring an app.
	 * \test onRefresh is still untested.
	*/
	virtual DECLSPEC void onRefresh();

private:
	std::string strFilename;
	ARGB_COLOR ulColorKey;
	char chType[4];
	bool bYMirror;// if true, the images are mirrored before copied to G-RAM
};

#endif
