/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLFONTSURFACES_H
#define SDLFONTSURFACES_H

#include "sdlsurfacecontainer.h"

/**
\ingroup sdl
\file sdlfontsurfaces.h
\brief Defines the class SdlFontSurfaces.
*/

/**
\ingroup sdl
\class SdlFontSurfaces
\brief Creates surfaces for SdlFont out of a ttf-file

With this class you can create surfaces for the SdlFont-class by using
a TrueTypeFont file.

\see SdlFont
*/
class DECLSPEC SdlFontSurfaces : public SdlSurfaceContainer
{
public:
	/**
	Constructor. Creates the surface container and draws the letters
	on the surfaces.
	\param ttf_file filename of the TrueTypeFont file
	\param letters the letters to write to the surfaces
	\param ptsize the size of the letters in points
	\param color the color, in which the font should be drawn
	\param name the name of the surface-container, see
	SdlSurfaceContainer::SdlSurfaceContainer()
	*/
	DECLSPEC SdlFontSurfaces( const char *ttf_file, const char *letters, int ptsize, SDL_Color color, const char *name = 0/*, SdlSurface *bmp = 0*/ );
	virtual DECLSPEC ~SdlFontSurfaces();

protected:
	/**
	Refreshes all surfaces, by reloading the font.
	This is neccessary, when restoring an application in MS Windows
	\todo test this (I think it does not function yet) and make it working.
	*/
	virtual DECLSPEC void onRefresh();

private:
	static unsigned int nRefCount;
	char *chFileName,
		*chLetters;
	int nPtSize;
	SDL_Color color;
//	SdlSurface *bgBmp;
};

#endif
