/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLSURFACECONTAINER_H
#define SDLSURFACECONTAINER_H

#include <vector>
#include "sdlmanager.h"
#include "sdlimage.h"

using namespace std;

/**
\ingroup sdl
\file sdlsurfacecontainer.h
\brief Defines the class SdlSurfaceContainer.
*/

/**
 * \ingroup sdl
 * \class SdlSurfaceContainer
 * \brief Base class for managing some surface resources for 2D-blitting.
 * 
 * This is the base class of a container of surface resources used for
 * 2D-blitting.
 */
class DECLSPEC SdlSurfaceContainer : public SdlImageContainer
{
public:
	/**
	Constructor. Registers itself in SdlManager with the given
	name.
	\param name the container's name
	\see SdlManager::getSurfacesByName()
	*/
	DECLSPEC SdlSurfaceContainer( const char *name );
	virtual DECLSPEC ~SdlSurfaceContainer();

	/**
	Gets a surface, which is stored in the container.
	\param i the number of the surface
	\returns a pointer to the selected surface or NULL, if
	the surface is not availible
	*/
	DECLSPEC SdlSurface *getSurface( unsigned int i );
	virtual SdlImage *getImage( unsigned int i )	{ return getSurface(i); }
	/// \returns the number of surfaces, stored in the container
	virtual unsigned int numImages()	{ return lSurfaces.size(); }

protected:
	/// list of all surfaces in the container
	vector<SdlSurface *> lSurfaces;
};

#endif
