/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef SDLTEXTURECONTAINER_H
#define SDLTEXTURECONTAINER_H

#include <vector>
#include "sdlmanager.h"
#include "sdltexture.h"

using namespace std;

/**
\ingroup sdl
\file sdltexturecontainer.h
\brief Defines the class SdlTextureContainer.
*/

/**
 * \ingroup sdl
 * \class SdlTextureContainer
 * \brief Base class for managing some texture resources for OpenGL.
 * 
 * This is the base class of a container of surface resources used for
 * drawing with OpenGL.
 */
class DECLSPEC SdlTextureContainer : public SdlImageContainer
{
public:
	/**
	 * Constructor.
	 * \param name the container's name
	 * \see SdlImageContainer::SdlImageContainer()
	 */
	DECLSPEC SdlTextureContainer( const char *name );
	virtual DECLSPEC ~SdlTextureContainer();

	/**
	 * Gets a texture, which is stored in the container.
	 * \param i the number of the texture
	 * \returns a pointer to the selected texture or NULL, if
	 * the texture is not availible
	 */
	DECLSPEC SdlTexture *getTexture( unsigned int i );
	virtual SdlImage *getImage( unsigned int i )	{ return getTexture(i); }
	/// \returns the number of textures, stored in the container
	virtual unsigned int numImages()	{ return lTextures.size(); }

protected:
	/// list of all textures in the container
	vector<SdlTexture *> lTextures;
};

#endif
