/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef UIFILETHEME_H
#define UIFILETHEME_H

#include "uitheme.h"
#include <list>

using namespace std;

class UiTranslator;

/**
\ingroup ui
\file uifiletheme.h
\brief Defines the class UiFileTheme.
*/

/**
\ingroup ui
\class UiFileTheme
\brief A theme-class, which loads its data from a text based file

This class makes it very easy to use and edit themes, because it loads the data
for look and feel from a text based file. This can easily be configured with any
text editor.\n
Also translating text is done by a text based file, using UiFileTranslator.

\todo create an UiXMLTheme class for reading themes from an XML based format.

\see ThemeManager, UiTranslator, UiFileTranslator
*/
class DECLSPEC UiFileTheme : public UiTheme
{
public:
	/**
	Constructor.
	\param fn the filename of the theme's configuration data
	\param name the theme's name
	\see UiTheme::UiTheme( const char * )
	*/
	DECLSPEC UiFileTheme( const char *fn, const char *name = 0 );
	virtual DECLSPEC ~UiFileTheme();

	virtual DECLSPEC const char *getString( const char *usage, int *length = 0 );
	virtual DECLSPEC Uint32 getUint32( const char *usage, bool *found = 0 );
	virtual DECLSPEC bool getBool( const char *usage, bool *found = 0 );
	virtual DECLSPEC SDL_Rect getRect( const char *usage );
	virtual DECLSPEC const char *translate( const char *text );

protected:
	virtual DECLSPEC Uint32 onLoad(bool hidden);
	virtual DECLSPEC Uint32 onActivate();
	virtual DECLSPEC Uint32 onDeactivate();
	virtual bool isLoaded()	{ return bLoaded; }

private:
	static unsigned int extractSubStrings( char *line, char **subs, unsigned int max );
	char *chFileName;

	struct INFO
	{
		char *usage;
		char *content;
		int lenght;
	};
	struct FONTINFO
	{
		int n;
		Uint32 cLight, cDark;
		char *chLightBmps, *chDarkBmps, *fnChars;
	};
/*	struct FONTSURFS
	{
		char chTTF[256], chChars[256], chName[256], chBgBmp[256];
		int nSize;
		SDL_Color color;
	};*/
	typedef list<INFO>::iterator infoIter;
	list<INFO> lInfos;

	UiTranslator *translator;

	bool bLoaded;
};

#endif
