/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBFILEPAGE_H
#define WEBFILEPAGE_H

#include <string>
#include "webpage.h"

/**
\ingroup web
\file webfilepage.h
\brief Defines the class WebFilePage.
*/

/**
\class WebFilePage
\ingroup web
\brief A class for simple publishing of a file in the web.

By creating a WebFilePage, you can make a file being availible
via HTTP.
*/
class DECLSPEC WebFilePage : public WebPage
{
public:
	/**
	Constructor. Automatically registers itself to be accessible
	by HTTP requests.
	\param uri the URI (after the IP address) of the page;
	when a HTTP request like http://IP-addr"uri" arrives, the function
	send() will be called. uri has to start with a '/'!
	\param fn the filename (relative to the app's data path) of the file,
	which should be sent to the requesting host.
	\param type the MIME-type, that should be sent to the requesting
	host (i.e. "text/html"), if NULL, the type is selected automatically
	by using the filename's ending.
	*/
	DECLSPEC WebFilePage( const char *uri, const char *fn, const char *type = NULL );
	virtual DECLSPEC ~WebFilePage();
	
	virtual DECLSPEC int send( WebSrvClient *client, Uint32 &responsecode );

protected:
	/// the filename of the file to send, when responding
	std::string strFilename;
	/// the content-type string
	char chContentType[128];
};

#endif
