/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBFORWARDPAGE_H
#define WEBFORWARDPAGE_H

#include "webpage.h"

/**
\ingroup web
\file webforwardpage.h
\brief Defines the class WebForwardPage.
*/

/**
\class WebForwardPage
\ingroup web
\brief A class for forwarding an URI to another URI.

This class forwards a request to another location by sending a
"Moved Permanently" response to the browser.
*/
class DECLSPEC WebForwardPage : public WebPage
{
public:
	/**
	Constructor. Automatically registers itself to be accessible
	by HTTP requests.
	\param uri the URI (after the IP address) of the page;
	when a HTTP request like http://IP-addr"uri" arrives, the function
	send() will be called. uri has to start with a '/' (slash)!
	\param location the URI of the file, where the browser should look
	for the page. Can be relative, like "index.html", or absolute, like
	"http://192.168.0.1/websrv/index.html".
	*/
	DECLSPEC WebForwardPage( const char *uri, const char *location );
	DECLSPEC ~WebForwardPage();
	
	/**
	Changes the location of the aim.
	\param l the URL to the aim. Can be absolute
	(i.e. "http://192.168.0.1/websrv/index.html") or relative
	(i.e. "index.html")
	*/
	DECLSPEC void setLocation( const char *l );
	virtual DECLSPEC int send( WebSrvClient *client, Uint32 &responsecode );

private:
	char *chLocation;
};

#endif
