/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBPAGE_H
#define WEBPAGE_H

#include <string>
#include "SDL/WebSrv.h"

/**
\ingroup web
\file webpage.h
\brief Defines the class WebPage.
*/

/**
\class WebPage
\ingroup web
\brief The basic class for publishing data in the web.

This class should not be used directly (it can, but it does not make
much sense). Derive it, or use the derived classes WebFilePage and
WebForwardPage.

This base class registers itself with the given URI at the WebServer-class
so that the virtual function send(WebSrvClient*) will be called, when a
request with the according URI is recieved.
*/
class DECLSPEC WebPage
{
public:
	/**
	Constructor.
	\param uri the URI, on which to respond (send(WebSrvClient*) will
	be called, when this uri is requested). uri always has to start
	with a '/'!
	*/
	DECLSPEC WebPage( const char *uri );
	DECLSPEC ~WebPage();
	
	/**
	Use this function in derived classes to respond on requests.
	\param client the connected client, that is requesting
	\param responsecode the return-code of the page, sent to the client
	(i.e. 200, when OK; 400 on error...) has to be assigned to this variable
	\return the number of bytes sent to the client
	*/
	virtual DECLSPEC int send( WebSrvClient *client, Uint32 &responsecode );
	/**
	\returns the URI, with which the class was created
	\see WebPage( const char * )
	*/
	virtual const char *getURI() const	{ return chURI; }

protected:
	/**
	This is a useful simple function to respond to a client with a string.
	\param client the client to respond to
	\param str the NULL-terminated string, that should be sent to the client
	\returns the number of bytes sent.
	*/
	static inline int sendString( WebSrvClient *client, const char *str )
	{
		return SDLNet_TCP_Send( client->sock, (char *)str, strlen( str ) );
	}
	/**
	Changes the URI on which to respond.
	\param uri the new URI to respond on, is has to begin with a '/'
	*/
	DECLSPEC void setURI( const char *uri );
	
private:
	char *chURI;
};

#endif
