/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBSTATISTICSPAGE_H
#define WEBSTATISTICSPAGE_H

#include "webpage.h"
#include <list>
#include "SDL/WebSrv.h"

using namespace std;

/**
\ingroup web
\file webstatisticspage.h
\brief Defines the class WebStatisticsPage.
*/

/**
\class WebStatisticsPage
\ingroup web
\brief A class for showing game statistics.

This class shows a simple webpage with game-statistics
It's very dirty code, because it's just for testing the network-interface or other things

*/
class DECLSPEC WebStatisticsPage : public WebPage
{
public:
	/// the result of the callback will be freed (deleted) automatically after usage!
	typedef char * (*Callback)( WebStatisticsPage *sender, void *data, const char *title, WebSrvClient * );
	struct TABLE_ROW
	{
		TABLE_ROW()
		{
			chTitle = NULL;
			cb = NULL;
			cbData = NULL;
		}
		TABLE_ROW( const TABLE_ROW &r )
		{
			if( r.chTitle )	{
				chTitle = new char[strlen(r.chTitle)+1];
				strcpy( chTitle, r.chTitle );
			}
			else chTitle = NULL;
			cb = r.cb;
			cbData = r.cbData;
		}
		~TABLE_ROW()
		{
			if( chTitle )
				delete[] chTitle;
		}
		char *chTitle;
		void *cbData;
		Callback cb;
	};
	typedef list<TABLE_ROW>::iterator ItemIter;

	DECLSPEC WebStatisticsPage( const char *uri );
	DECLSPEC ~WebStatisticsPage();

	virtual DECLSPEC int send( WebSrvClient *client, Uint32 &responsecode );// returns the number of bytes sent to client

	DECLSPEC ItemIter addItem( const char *title, Callback c, void *cb_data );
	DECLSPEC void removeItem( ItemIter i );

	DECLSPEC void setTitle( const char *t );
	DECLSPEC void setHeading( const char *t );
	DECLSPEC void setEnding( const char *t );
	
	int wTableTitles,// in percent
		wTable, wBorder;
	Uint32 colorTable;

private:
	char *chTitle, *chHeading, *chEnding;
	list<TABLE_ROW> lItems;
};

#endif
