/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#ifndef WEBTHREAD_H
#define WEBTHREAD_H

#include "SDL/WebSrv.h"
#include "basicthread.h"

/**
\ingroup web
\file webthread.h
\brief Defines the class WebThread.
*/

/**
\class WebThread
\ingroup web
\brief The WebThread class is used to respond on web requests asynchronously.

Usually you do not need this class in your app. It is only required interally by the
Netrinjo engine.
*/
class DECLSPEC WebThread : public BasicThread
{
public:
	/**
	Constructor. Debugging log file does not make much sense.
	*/
	DECLSPEC WebThread( const char *name, BasicObject *p, const char * debug = NULL );
	DECLSPEC ~WebThread();
	
	/**
	The time of inactivity, in msecs, after that a thread can be killed.\n
	Default value: 30000
	*/
	static Uint32 tIdleToExit;
protected:
	virtual DECLSPEC int onRun();

private:
	Uint32 tLastAction;
	WebSrvClient *client;
	
	friend class WebServer;
};

#endif
