/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guicombination.h"
#include "sdlmanager.h"
#include "functions.h"

DECLARE_SLOT(new, GuiCheck, GuiCheck::newguiobject)
DECLARE_SLOT(new, GuiRadio, GuiRadio::newguiobject)
DECLARE_SLOT(new, GuiNumScroller, GuiNumScroller::newguiobject)


//////////////////////////////////////////////////////////////////////
// GuiCheck

GuiCheck::GuiCheck( const SDL_Rect &r, GuiElement *p, const char *name, const char *t )
	: GuiElement( r, p, name )
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText(tmp, this, "", t);
	check = new GuiCheckableBtn( tmp, this, "", NULL,
		SdlManager::the->getImagesByName( "check" ),
		SdlManager::the->getAudioByName( "click up" ),
		SdlManager::the->getAudioByName( "click down" ) );
	text->setAlign( SdlFont::baseline, SdlFont::left );
	arrange();
}

GuiCheck::GuiCheck( TiXmlElement *el, GuiElement *p ) : GuiElement(el, p)
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText(tmp, this, "", el->Attribute("caption"));
	check = new GuiCheckableBtn( tmp, this, "", NULL,
		SdlManager::the->getImagesByName( "check" ),
		SdlManager::the->getAudioByName( "click up" ),
		SdlManager::the->getAudioByName( "click down" ) );
}

GuiCheck::~GuiCheck()
{

}

void GuiCheck::onInit()
{
	arrange();
	text->setAlign( SdlFont::baseline, SdlFont::left );
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("Caption");
		if(str)
			text->setText(str);
	}
	GuiElement::onInit();
}

Uint32 GuiCheck::onResize()
{
	arrange();
	return 0;
}

void GuiCheck::arrange()
{
	Uint32 w, h;
	getWndSize(w, h);
	check->resizeTo(20, 20);
	int x = 10;
	int y = h / 2;
	check->moveRelCenterTo(x,y);
	SDL_Rect r = {20 + 10, 0, w - (20 + 10), h};
	text->setRelWindow(r);
}

//////////////////////////////////////////////////////////////////////
// GuiRadio

GuiRadio::GuiRadio(const SDL_Rect &r, GuiRadioContainer *p, const char *name,const char *t) : GuiElement( r, p, name )
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText(tmp,this,"",t);
	radio = new GuiRadioBtn( tmp, p, "", NULL,
		SdlManager::the->getImagesByName( "check" )/*,
		SdlManager::the->getAudioByName( "up click" ),
		SdlManager::the->getAudioByName( "down click" )*/ );
	text->setAlign( SdlFont::baseline, SdlFont::left );
	arrange();
}

GuiRadio::GuiRadio( TiXmlElement *el, GuiRadioContainer *p ) : GuiElement(el, p)
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText(tmp,this,"",el->Attribute("Caption"));
	radio = new GuiRadioBtn( tmp, p, "", NULL,
		SdlManager::the->getImagesByName( "check" )/*,
		SdlManager::the->getAudioByName( "up click" ),
		SdlManager::the->getAudioByName( "down click" )*/ );
	text->setAlign( SdlFont::baseline, SdlFont::left );
	arrange();
}

GuiRadio::~GuiRadio()
{
}

void GuiRadio::onInit()
{
	arrange();
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiElement::onInit();
}

void GuiRadio::destroy()
{
	delete radio;
}

Uint32 GuiRadio::onResize()
{
	arrange();
	return 0;
}

void GuiRadio::arrange()
{
	Uint32 w,h;
	getWndSize( w, h );
	SDL_Rect rel = getRelWindow();
	const int middle = 20;
	int cx = middle, cy = 20;
	radio->resizeTo( cx, cy );
	cy = h / 2 + rel.y;
	cx = cx / 2 + rel.x;
	radio->moveRelCenterTo( cx, cy );
	SDL_Rect r = { middle + 10, 0, w - (middle + 10), h };
	text->setRelWindow(r);
}


//////////////////////////////////////////////////////////////////////
// GuiNumScroller

GuiNumScroller::GuiNumScroller(const SDL_Rect &r, GuiElement *p, const char *name) : GuiElement( r, p, name )
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText( tmp, this, "" );
	scroller = new GuiScroller( tmp, this, "" );
	scroller->setDirection( GuiScroller::horizontal );
	scroller->setCallbackData( this );
	scroller->setChangeCallback( onChange );
	arrange();
	onChange();
}

GuiNumScroller::GuiNumScroller(TiXmlElement *el, GuiElement *p) : GuiElement( el, p )
{
	SDL_Rect tmp = {0,0,0,0};
	text = new GuiText( tmp, this, "" );
	scroller = new GuiScroller( tmp, this, "" );
	scroller->setDirection( GuiScroller::horizontal );
	scroller->setCallbackData( this );
	scroller->setChangeCallback( onChange );
	arrange();
	onChange();
}

GuiNumScroller::~GuiNumScroller()
{
}

void GuiNumScroller::onInit()
{
	arrange();
	onChange();
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiElement::onInit();
}

Uint32 GuiNumScroller::onResize()
{
	arrange();
	return 0;
}

void GuiNumScroller::arrange()
{
	Uint32 w,h;
	getWndSize( w, h );
	const int middle = 60;
	SDL_Rect r = { 0, 0, middle, h };
	text->setRelWindow( r );
	r.x = middle;
	r.y = 0;
	r.w = w - middle;
	r.h = h;
	scroller->setRelWindow( r );
}

void GuiNumScroller::onChange()
{
	text->setText( intToStr( scroller->getScrollPos(), 10 ) );
}

void GuiNumScroller::updatetext()
{
	onChange();
}
