/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guicursor.h"
#include "guimanager.h"
#include "basicapp.h"

DECLARE_SLOT(new, GuiCursor, GuiCursor::newguiobject)

GuiCursor::GuiCursor( const SDL_Rect &r, GuiElement *p, const char *name, SdlImage *bmp, Uint16 x, Uint16 y )
: GuiPictureBg( r, p, name, CLR_INVALID, bmp )
{
	setVisible(false);
	xHot = x;
	yHot = y;
	if(GuiManager::the)
		GuiManager::the->setCursor(this);
}

GuiCursor::GuiCursor( TiXmlElement *el, GuiElement *p ): GuiPictureBg( el, p )
{
	xHot = 0;
	yHot = 0;
	if(GuiManager::the)
		GuiManager::the->setCursor(this);
}

GuiCursor::~GuiCursor()
{

}

void GuiCursor::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		if(el->QueryIntAttribute("HotX", &xHot) != TIXML_SUCCESS)
			xHot = 0;
		if(el->QueryIntAttribute("HotY", &yHot) != TIXML_SUCCESS)
			yHot = 0;
	}
	GuiPictureBg::onInit();
}

Uint32 GuiCursor::onMouseMove( Uint32 x, Uint32 y )
{
	if(!isVisible())
		setVisible(true);
	
	SDL_Rect r = getRelWindow();
	r.x = x - xHot;
	r.y = y - yHot;
	setRelWindow(r);
	return 0;
}

