/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guidropdown.h"
#include <string>
#include "guitext.h"
#include "guidynamicbutton.h"
#include "thememanager.h"
#include "uitheme.h"
#include "functions.h"
#include "basicapp.h"

DECLARE_SLOT(new, GuiDropDown, GuiDropDown::newguiobject)

GuiDropDown::GuiDropDown( const SDL_Rect &r, GuiElement *p, const char *name, bool editable ): GuiElement( r, p, name )
{
	SDL_Rect rr = r;
	rr.y += 20;
	rr.h -= 20;
	gList = new GuiList( rr, p, "" );
	gList->setCallbackData( this );
	gList->setSelChangeCallback( onList );
	gList->setVisible( false );
	rr.y = r.y;
	rr.h = 20;
	setRelWindow( rr );
	rr.x = rr.y = 0;
	gpBg = new GuiPictureBg( rr, this, "" );
	rr.w -= 20;
	if( editable )	{
		gText = 0;
		gtInput = new GuiTextInput( rr, gpBg, "" );
	}
	else	{
		gtInput = 0;
		gText = new GuiText( rr, gpBg, "" );
	}
	rr.x += rr.w;
	rr.w = 20;
	gcButton = new GuiCheckableBtn( rr, this, "" );
	gcButton->setCallbackData( this );
	gcButton->setCheckCallback( onButton );

	if( ThemeManager::the )
		GuiDropDown::onThemeChanged( ThemeManager::the->getActiveTheme() );
}

GuiDropDown::GuiDropDown( TiXmlElement *el, GuiElement *p ): GuiElement( el, p )
{
	SDL_Rect r = getRelWindow();
	SDL_Rect rr = r;
	rr.y += 20;
	rr.h -= 20;
	gList = new GuiList( rr, p, "" );
	gList->setCallbackData( this );
	gList->setSelChangeCallback( onList );
	gList->setVisible( false );
	rr.y = r.y;
	rr.h = 20;
	setRelWindow( rr );
	rr.x = rr.y = 0;
	gpBg = new GuiPictureBg( rr, this, "" );
	rr.w -= 20;
	const char *str = el->Attribute("Editable");
	if( str && strToBool(str)  )	{
		gText = 0;
		gtInput = new GuiTextInput( rr, gpBg, "" );
	}
	else	{
		gtInput = 0;
		gText = new GuiText( rr, gpBg, "" );
	}
	rr.x += rr.w;
	rr.w = 20;
	gcButton = new GuiCheckableBtn( rr, this, "" );
	gcButton->setCallbackData( this );
	gcButton->setCheckCallback( onButton );
}

GuiDropDown::~GuiDropDown()
{
}

void GuiDropDown::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiElement::onInit();
}

void GuiDropDown::destroy()
{
	delete gList;
	gList = 0;
}

const char *GuiDropDown::getString() const
{
	if( gtInput )
		return gtInput->getText();
	if( gText )
		return gText->getText();
	return "";// will never be reached
}

void GuiDropDown::onButton()
{
	if( gList->isVisible() )
		gList->setVisible( false );
	else
		gList->setVisible( true );
}

void GuiDropDown::onList()
{
	if( gtInput )
		gtInput->setText( gList->getSelStr() );
	if( gText )
		gText->setText( gList->getSelStr() );

	gList->setVisible( false );
}

Uint32 GuiDropDown::onResize()
{
	SDL_Rect rr = getRelWindow();
	if( rr.h == 20 )
		return GuiElement::onResize();

	rr.y += 20;
	rr.h -= 20;
	gList->setRelWindow( rr );

	rr.y -= 20;
	rr.h = 20;
	setRelWindow( rr );
	rr.x = rr.y = 0;
	gpBg->setRelWindow( rr );
	rr.w -= 20;
	if( gtInput )
		gtInput->setRelWindow( rr );
	if( gText )
		gText->setRelWindow( rr );

	rr.x += rr.w;
	rr.w = 20;
	gcButton->setRelWindow( rr );

	return GuiElement::onResize();
}

Uint32 GuiDropDown::onThemeChanged( UiTheme *t )
{
	if( t == 0 )
		return GuiElement::onThemeChanged( t );

	const char *chName;
	gpBg->bgcolor = t->getUint32( ((string)getClassName() + "->background-color" ).c_str() );
	chName = t->getString( ((string)getClassName() + "->background-bmp" ).c_str() );
	if( chName )
		gpBg->setBmp( SdlManager::the->getImageByName( chName ) );

	gList->bgcolor = t->getUint32( ((string)getClassName() + "->background-color-box" ).c_str() );
	chName = t->getString( ((string)getClassName() + "->background-bmp-box" ).c_str() );
	if( chName )
		gList->setBmp( SdlManager::the->getImageByName( chName ) );

	chName = t->getString( ((string)getClassName() + "->button-bmps" ).c_str() );
	if( chName )
		gcButton->setButtonBmps( SdlManager::the->getImagesByName( chName ) );
	
	gText->setFont( SdlFont::get( t->getUint32( ((string)getClassName() + "->font-size" ).c_str() ),
								t->getBool( ((string)getClassName() + "->font-light" ).c_str() )) );
	
	return GuiElement::onThemeChanged( t );
}
