/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guifileelement.h"
#include <string>
#include "tinyxml.h"
#include "functions.h"
#include "basicapp.h"

using namespace std;

DECLARE_SLOT(new, GuiFileElement, GuiFileElement::newguiobject)

GuiFileElement::GuiFileElement( SDL_Rect &r, GuiElement *el, const char *name, const char *filename )
: GuiElement( r, el, name )
{
	if(filename)	{
		if(xmlChildrenDoc.LoadFile(filename) && xmlChildrenDoc.RootElement())
			create();
	}
}

GuiFileElement::GuiFileElement( TiXmlElement *el, GuiElement *p )
: GuiElement( el, p )
{
	const char *filename = el->Attribute("Filename");
	if(filename)
	{
		if(xmlChildrenDoc.LoadFile(BasicApp::the->makeDataPath(filename).c_str()) && xmlChildrenDoc.RootElement())
		{
			create();
		}
	}
}

GuiFileElement::~GuiFileElement()
{
}

void GuiFileElement::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiElement::onInit();
}

bool GuiFileElement::initByFile(const char *fn)
{
	// clean up
	while(!children.empty())
		delete children.front();
	// try to load new configuration
	if(fn)	{
		if(xmlChildrenDoc.LoadFile(BasicApp::the->makeDataPath(fn).c_str()) && xmlChildrenDoc.RootElement())	{
			create();
			GuiElement::onInit();
			return true;
		}
	}
	return false;
}

int GuiFileElement::create()
{
	if(!xmlChildrenDoc.RootElement())
		return -1;
	if( strcmp( getClassName(), xmlChildrenDoc.RootElement()->Value() ) != 0 )
	{
		/*
		BasicApp::the->logLevelLine("WARNING: GuiFileElement:create: root element is not '");
		BasicApp::the->logString( getClassName() );
		BasicApp::the->logString("', this misconfiguration is ignored.");
		*/
	}
	int n = 0;
	for(TiXmlElement *child = xmlChildrenDoc.RootElement()->FirstChildElement(); child; child = child->NextSiblingElement())
	{
		GuiElement::create(child, this);
		n++;
	}
	return n;
}
