/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guihint.h"
#include <string>
#include "thememanager.h"
#include "definitions.h"

DECLARE_SLOT(new, GuiHint, GuiHint::newguiobject)

GuiHint::GuiHint( const SDL_Rect &r, GuiElement *el, const char *name ): GuiFlowWindow( r, el, name, CLR_STANDARD )
{
	setVisible( false );
	tShow = 1000;
	SDL_Rect rr = r;
	rr.x = rr.y = 0;
	text = new GuiText( rr, this, "" );
	timerinfo = FrameTimer::the->addCallback( onTimer, this, INFINITE );

	GuiHint::onThemeChanged( ThemeManager::the ? ThemeManager::the->getActiveTheme() : 0 );
}

GuiHint::GuiHint( TiXmlElement *el, GuiElement *p ): GuiFlowWindow( el, p )
{
	setVisible( false );
	tShow = 1000;
	SDL_Rect rr = getRelWindow();
	rr.x = rr.y = 0;
	text = new GuiText( rr, this, "" );
	timerinfo = FrameTimer::the->addCallback( onTimer, this, INFINITE );
}

GuiHint::~GuiHint()
{
}

void GuiHint::onInit()
{
	/*TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiFlowWindow::onInit();
}

void GuiHint::show(int time)
{
	if( time > 0 )
		FrameTimer::the->setInterval( timerinfo, time );
	else
		FrameTimer::the->setInterval( timerinfo, tShow );
	setVisible( true );
}

Uint32 GuiHint::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiFlowWindow::onThemeChanged( t );
	if( t == 0 )
		return r;

	tShow = t->getUint32( ( string(getClassName()) + "->showtime" ).c_str() );
	text->chooseFont( (GuiText::FontSize)t->getUint32( ( string(getClassName()) + "->font" ).c_str() ), false );
	text->setAlign( (SdlFont::VAlign)t->getUint32( ( string(getClassName()) + "->valign" ).c_str() ), (SdlFont::HAlign)t->getUint32( ( string(getClassName()) + "->halign" ).c_str() ) );

	return r;
}

void GuiHint::onTimer()
{
	FrameTimer::the->setInterval( timerinfo, INFINITE );
	setVisible( false );
}

void GuiHint::showHint( const char *txt, int time, SDL_Rect *r )
{
	if( r != 0 )
		setAbsWindow( *r );

	text->setText( txt );
	show( time );
}

Uint32 GuiHint::onResize()
{
	SDL_Rect r = getRelWindow();
	r.x = r.y = 0;
	text->setRelWindow( r );
	return GuiFlowWindow::onResize();
}
