/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guilistbox.h"
#include "guitext.h"
#include "guidynamicbutton.h"
#include "functions.h"

DECLARE_SLOT(new, GuiListBox, GuiListBox::newguiobject)

GuiListBox::GuiListBox( const SDL_Rect &r, GuiElement *p, const char *name, bool editable ): GuiElement( r, p, name )
{
	SDL_Rect rr = r;
	rr.y += 20;
	rr.h -= 20;
	gList = new GuiList( rr, p, "" );
	gList->setCallbackData( this );
	gList->setSelChangeCallback( onList );
	rr.y = r.y;
	rr.h = 20;
	setRelWindow( rr );
	rr.x = rr.y = 0;
	rr.w -= 20;
	if( editable )	{
		gText = 0;
		gtInput = new GuiTextInput( rr, this, "" );
	}
	else	{
		gtInput = 0;
		gText = new GuiText( rr, this, "" );
	}
	rr.x += rr.w;
	rr.w = 20;
	gcButton = new GuiClickableBtn( rr, this, "" );
	gcButton->setCallbackData( this );
	gcButton->setClickCallback( onButton );
}

GuiListBox::GuiListBox( TiXmlElement *el, GuiElement *p ): GuiElement( el, p )
{
	SDL_Rect rr, r = getRelWindow();
	rr = r;
	rr.y += 20;
	rr.h -= 20;
	gList = new GuiList( rr, p, "" );
	gList->setCallbackData( this );
	gList->setSelChangeCallback( onList );
	rr.y = r.y;
	rr.h = 20;
	setRelWindow( rr );
	rr.x = rr.y = 0;
	rr.w -= 20;
	const char *str = el->Attribute("Editable");
	if( str && strToBool(str)  )	{
		gText = 0;
		gtInput = new GuiTextInput( rr, this, "" );
	}
	else	{
		gtInput = 0;
		gText = new GuiText( rr, this, "" );
	}
	rr.x += rr.w;
	rr.w = 20;
	gcButton = new GuiClickableBtn( rr, this, "" );
	gcButton->setCallbackData( this );
	gcButton->setClickCallback( onButton );
}

GuiListBox::~GuiListBox()
{

}

void GuiListBox::onInit()
{
/*	TiXmlElement *el = getXmlData();
	if(el)	{
	}*/
	GuiElement::onInit();
}

void GuiListBox::destroy()
{
	delete gList;
	gList = 0;
}

void GuiListBox::onButton()
{
	if( gList->isVisible() )
		gList->setVisible( false );
	else
		gList->setVisible( true );
}

void GuiListBox::onList()
{
	if( gtInput )
		gtInput->setText( gList->getSelStr() );
	if( gText )
		gText->setText( gList->getSelStr() );

	gList->setVisible( false );
}


