/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <string>
#include <GL/glu.h>
#include "guiprogressbar.h"
#include "guimanager.h"
#include "thememanager.h"
#include "basicapp.h"
#include "sdlfiletexture.h"
#include "sdlfilesurface.h"
#include "rectfunctions.h"
#include "functions.h"
#include "oglfunctions.h"
#include "definitions.h"

SdlImage *GuiProgressBar::waitres = NULL;

DECLARE_SLOT(new, GuiProgressBar, GuiProgressBar::newguiobject)

GuiProgressBar::GuiProgressBar( const SDL_Rect &r, GuiElement *el, const char *name, SdlImage *progbmp, SdlImage *backbmp )
	: GuiPictureBg( r, el, name, 0, backbmp )
{
	stepsize = 10;
	maxvalue = 100;
	progress = 0;
	horizontal = true;
	bmp = progbmp;

	bThemeBmp = progbmp == 0;
	bThemeFont = true;
	bThemeVal = true;
	bThemeHorz = true;

	SDL_Rect rr = r;
	rr.x = rr.y = 0;
	brd = new GuiBorder( rr, this, "" );
	brd->invert = true;
	text = new GuiText( rr, brd, "", "0%" );
	text->chooseFont( GuiText::fs_text, true );

	GuiProgressBar::onThemeChanged( ThemeManager::the ? ThemeManager::the->getActiveTheme() : 0 );
}

GuiProgressBar::GuiProgressBar( TiXmlElement *el, GuiElement *p ): GuiPictureBg( el, p )
{
	stepsize = 10;
	maxvalue = 100;
	progress = 0;
	horizontal = true;
	bmp = NULL;

	bThemeBmp = bmp == 0;
	bThemeFont = true;
	bThemeVal = true;
	bThemeHorz = true;

	SDL_Rect rr = getRelWindow();
	rr.x = rr.y = 0;
	brd = new GuiBorder( rr, this, "" );
	brd->invert = true;
	text = new GuiText( rr, brd, "", "0%" );
	text->chooseFont( GuiText::fs_text, true );
}

GuiProgressBar::~GuiProgressBar()
{

}

void GuiProgressBar::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("ProgressBmp");
		if(str)	{
			bmp = SdlManager::the->getImageByName(str);
		}
	}
	GuiPictureBg::onInit();
}

Uint32 GuiProgressBar::onFrame( SdlSurface *dst )
{
	paintBg( dst );
	if( bmp == 0 )
		return GuiPictureBg::onFrame( dst );

	Sint32 x, y;
	SDL_Rect r = brd->getDrawWindow();
	x = r.x + r.w/2;
	y = r.y + r.h/2;
	if( horizontal )	{
		x -= (stepsize * maxvalue) / 2;
		y -= bmp->getHeight() / 2;
	}
	else	{
		y -= (stepsize * maxvalue) / 2;
		x -= bmp->getWidth() / 2;
	}

	dst->setClip( getClip() );

	for( int i = 0; i < progress * width / (maxvalue * stepsize); i++ )	{
		if( horizontal )
			x += stepsize;
		else
			y += stepsize;

		dst->fastBlit( (SdlSurface*)bmp, x, y, 0 );
	}
	return GuiPictureBg::onFrame( dst );
}

Uint32 GuiProgressBar::onGLFrame()
{
	paintBgGL();
	if( bmp == 0 )
		return GuiPictureBg::onGLFrame();

/*	Sint32 x, y; TODO
	SDL_Rect r = brd->getDrawWindow();
	x = r.x + r.w/2;
	y = r.y + r.h/2;
	if( horizontal )	{
		x -= (stepsize * maxvalue) / 2;
		y -= bmp->getHeight() / 2;
	}
	else	{
		y -= (stepsize * maxvalue) / 2;
		x -= bmp->getWidth() / 2;
	}

	dst->setClip( getClip() );

	for( int i = 0; i < progress * width / (maxvalue * stepsize); i++ )	{
		if( horizontal )
			x += stepsize;
		else
			y += stepsize;

		dst->fastBlit( (SdlSurface*)bmp, x, y, 0 );
	}*/
	return GuiPictureBg::onGLFrame();
}

Uint32 GuiProgressBar::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiPictureBg::onThemeChanged( t );
	if( t == 0 )
		return r;

	if( bThemeBmp )	{
		const char *name = t->getString( ( string(getClassName()) + "->progress-bmp" ).c_str() );
		if( name )	{
			bmp = SdlManager::the->getImageByName( name );
		}
		else bmp = 0;
	}
	if( bThemeFont )	{
		text->setAlign( (GuiText::VAlign)t->getUint32( ( string(getClassName()) + "->valign" ).c_str() ),
			(GuiText::HAlign)t->getUint32( ( string(getClassName()) + "->halign" ).c_str() ) );

		text->chooseFont( (GuiText::FontSize)t->getUint32( ( string(getClassName()) + "->fontsize" ).c_str() ),
			t->getBool( ( string(getClassName()) + "->font-light" ).c_str() ) );
	}
	if( bThemeHorz )	{
		bool succ;
		horizontal = t->getBool( ( string(getClassName()) + "->horizontal" ).c_str(), &succ );
		if( !succ )
			horizontal = true;
	}
	if( bThemeVal )	{
		stepsize = t->getUint32( ( string(getClassName()) + "->stepsize" ).c_str() );
		if( stepsize == 0 )
			stepsize = 10;
		maxvalue = t->getUint32( ( string(getClassName()) + "->maxvalue" ).c_str() );
		if( maxvalue == 0 )
			maxvalue = 100;
	}

	return r;
}

void GuiProgressBar::arrange()
{
	SDL_Rect r = getAbsWindow();
	r.x = r.y = 0;
	brd->setRelWindow( r );
	width = horizontal ? r.w - 6 : r.h - 6;
}

void GuiProgressBar::setValues( int max, int step )
{
	maxvalue = max;
	stepsize = step;
	bThemeVal = false;
}

void GuiProgressBar::setHorizontal( bool h )
{
	bThemeHorz = false;
	horizontal = h;
}


Uint32 GuiProgressBar::onResize()
{
	arrange();
	return GuiPictureBg::onResize();
}

int GuiProgressBar::operator ++()
{
	if( progress < maxvalue )
		progress++;
	char * txt = new char[5];
	strcpy( txt, intToStr( progress, 10 ) );
	strcat( txt, "%" );
	text->setText( txt );
	delete[] txt;

	return progress;
}

int GuiProgressBar::operator += ( int i )
{
	if( progress <= maxvalue - i )
		progress += i;
	char txt[5];
	strcpy( txt, intToStr( progress, 10 ) );
	strcat( txt, "%" );
	text->setText( txt );

	return progress;
}

void GuiProgressBar::setProgress(int i)
{
	progress = MAX( i, maxvalue );
	char txt[5];
	strcpy( txt, intToStr( i, 10 ) );
	strcat( txt, "%" );
	text->setText( txt );
}

void GuiProgressBar::setFont( SdlFont *f )
{
	bThemeFont = false;
	text->setFont( f );
}

void GuiProgressBar::doFrame( SdlSurface *dst )
{
	onFrame( dst );
}

bool GuiProgressBar::initsimple( const char *waitbmp )
{
	BasicApp::the->logBegin( "Initializing simple progressbar..." );
// surface-bitmap evtl. noch nicht geladen
	SDL_Rect r = SdlManager::the->getScreenRect();

	if( SdlManager::the->getImageByName( waitbmp ) == 0 )
	{
		string strFN = BasicApp::the->makeDataPath( waitbmp );
		if( SdlManager::the->isUsingOpenGL() )
		{
			BasicApp::the->logLevelLine( "Using OpenGL." );
			SdlFileTexture *t = new SdlFileTexture( strFN.c_str(), waitbmp, 0xFFFFFF );
			if( t->getId() != 0 )
				BasicApp::the->logLevelLine( ( string(waitbmp) + " loaded" ).c_str() );
			else	{
				BasicApp::the->logLevelLine( "ERROR: cannot init simple progressbar" );
			}
			GuiManager::resetGLstates( r.w, r.h );
			glEnable( GL_TEXTURE_2D );
			glClear( GL_COLOR_BUFFER_BIT );
		}
		else
		{
			BasicApp::the->logLevelLine( "Using 2D-blitting." );
			SdlFileSurface *s = new SdlFileSurface( strFN.c_str(), waitbmp, 0xFFFFFF );
			if( s->surf() != 0 )
				BasicApp::the->logLevelLine( ( string(waitbmp) + " loaded" ).c_str() );
			else	{
				BasicApp::the->logLevelLine( "ERROR: cannot init simple progressbar" );
			}
		}
	}

	waitres = SdlManager::the->getImageByName( waitbmp );

//	SdlSurface *d = SdlManager::the->getDisplay();
//	d->fill( &r, RGB( 0, 0, 0 ) );
//	showprogresssimple( 0.f );
	// and delete a 2nd time, because of double backbuffer
//	d->fill( &r, RGB( 0, 0, 0 ) );

	BasicApp::the->logEnd( "done." );

	return waitres != 0;
}

void GuiProgressBar::uninitsimple()
{
	if( waitres )
		delete waitres;
	waitres = 0;
	BasicApp::the->logLevelLine( "Simple progressbar uninitialized." );
}

void GuiProgressBar::showprogresssimple( float f, ARGB_COLOR barcolor, ARGB_COLOR bordercolor, ARGB_COLOR emptycolor )
{
	if( !waitres )
		return;
	
	SDL_Rect r = SdlManager::the->getScreenRect();
	SdlSurface *d = SdlManager::the->isUsingOpenGL() ? NULL : SdlManager::the->getDisplay();

	int wWait, hWait;
	if( waitres )	{
		waitres->getSize( wWait, hWait );
	}
	else	{
		wWait = 0;
		hWait = 0;
	}

	if( d )
		d->fastBlit( (SdlSurface *)waitres, ( r.w - wWait )/2, ( r.h - hWait )/2, 0 );
	else
	{// use OpenGL
		SDL_Rect dest = {( r.w - wWait )/2, ( r.h - hWait )/2, wWait, hWait};
		((SdlTexture*)waitres)->drawSimple( dest );
	}

	int x = r.w, y = r.h;
	x >>= 1;
	y >>= 1;
	r.y = y + hWait;
	r.x = x;
	r.w = r.h = 0;
	inflateRect( r, 100, 20 );
	if( d )
		d->fill( &r, bordercolor );
	else
		ogl::fillRect( r, bordercolor );
	inflateRect( r,-5,-5 );
	if( d )
		d->fill( &r, emptycolor);
	else
		ogl::fillRect( r, emptycolor );

	r.w = (Uint16)(f * r.w);
	if( d )
		d->fill( &r, barcolor );
	else
		ogl::fillRect( r, barcolor );
	
	if( d )
		SDL_Flip( d->surf() );
	//	SDL_UpdateRect( d->surf(), 0,0,0,0 );
	else
		SDL_GL_SwapBuffers();
}
