/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999 - 2007 by Tobias Nadler and Georg Schnattinger           */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guirotator.h"
#include "SDL/SDL_rotozoom.h"
#include <math.h>
#include "sdlsurface.h"
#include "sdltexture.h"
#include "sdlmanager.h"
#include "functions.h"
#include "basicapp.h"

DECLARE_SLOT(new, GuiRotator, GuiRotator::newguiobject)

GuiRotator::GuiRotator( const SDL_Rect &r, GuiElement *p, const char *name, SdlImage *bmp, double angle )
: GuiElement( r, p, name )
{
	dAngle = angle;
	dSpeed = M_PI;
	sBmp = bmp;
	dZoom = 1;
	bSmooth = false;
}

GuiRotator::GuiRotator( TiXmlElement *el, GuiElement *p )
: GuiElement( el, p )
{
	dAngle = 0;
	dSpeed = 0;
	dZoom = 1;
	bSmooth = false;
	
	sBmp = NULL;
}

GuiRotator::~GuiRotator()
{

}

void GuiRotator::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char * str = el->Attribute("Angle");
		if(str)
			dAngle = atof(str);
		str = el->Attribute("Speed");
		if(str)
			dSpeed = atof(str);
		str = el->Attribute("Zoom");
		if(str)
			dZoom = atof(str);
		str = el->Attribute("Smooth");
		bSmooth = str && strToBool(str);
		
		str = el->Attribute("RotatingBmp");
		if(str)
			sBmp = SdlManager::the->getImageByName(str);
	}
	GuiElement::onInit();
}

void GuiRotator::setAngleSpeed( double a, double speed )
{
	dAngle = a;
	dSpeed = speed;
	nTime = SDL_GetTicks();
}

void GuiRotator::setAngle( double a )
{
	dAngle = a;
	nTime = SDL_GetTicks();
}

void GuiRotator::setSpeed( double speed )
{
	dAngle += dSpeed * ( SDL_GetTicks() - nTime ) / 1000;
	dSpeed = speed;
	nTime = SDL_GetTicks();
}


Uint32 GuiRotator::onFrame( SdlSurface *dst )
{
	Uint32 r = GuiElement::onFrame( dst );
	paintBg( dst );
	return r;
}

Uint32 GuiRotator::onGLFrame()
{
	Uint32 r = GuiElement::onGLFrame();
	paintBgGL();
	return r;
}

void GuiRotator::paintBg( SdlSurface *s )
{
	if( sBmp == 0 )
		return;

	SDL_Rect r = getAbsWindow();
	double a = dAngle + dSpeed * ( SDL_GetTicks() - nTime ) / 1000;
	a *= M_PI / 180.0;
	s->setClip( getClip() );
	s->rotozoomBlit( (SdlSurface*)sBmp, r.x+r.w/2, r.y+r.h/2, a, dZoom, bSmooth );
}

void GuiRotator::paintBgGL()
{
	if( sBmp == 0 )
		return;

	SDL_Rect r = getAbsWindow();
	double a = dAngle + dSpeed * ( SDL_GetTicks() - nTime ) / 1000;
	((SdlTexture*)sBmp)->rotoDrawClipped( getClip(), r.x+r.w/2, r.y+r.h/2, a, dZoom );
}

