/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <math.h>
#include <string>
#include "guiscroller.h"
#include "sdlmanager.h"
#include "basicapp.h"
#include "thememanager.h"
#include "definitions.h"
#include "functions.h"

DECLARE_SLOT(new, GuiScroller, GuiScroller::newguiobject)
DECLARE_SLOT(new, GuiScrollBg, GuiScrollBg::newguiobject)

GuiScroller::GuiScroller( const SDL_Rect &r, GuiElement *p, const char *name, bool usetheme ): GuiPictureBg( r, p, name )
{
	bThemeBtns = usetheme;
	SDL_Rect tmp = {0,0,0,0};

	buttonincrement = new GuiClickableBtn( tmp, this, "" );
	buttondecrement = new GuiClickableBtn( tmp, this, "" );
	buttonmove = new GuiMoveableBtn( tmp, this, "" );

	buttonincrement->setBorderVisible( false );
	buttondecrement->setBorderVisible( false );
	buttonmove->setBorderVisible( false );

	buttonincrement->setCallbackData( this );
	buttondecrement->setCallbackData( this );
	buttonmove->setCallbackData( this );
	buttonincrement->setClickCallback( &onButton );
	buttondecrement->setClickCallback( &onButton );
	buttonmove->setPosChangeCallback( &onButton, this );
	buttonmove->setClickCallback( &onButton );

	max = 0;
	min = 0;
	tick = 1;
	pos = 0;
	dir = r.w < r.h ? vertikal : horizontal;
	cbChange.func = 0;

	arrange();
	GuiScroller::onThemeChanged( ThemeManager::the ? ThemeManager::the->getActiveTheme() : 0 );
}

GuiScroller::GuiScroller( TiXmlElement *el, GuiElement *p ): GuiPictureBg( el, p )
{
	bThemeBtns = true;
	cbChange.func = 0;
	
	SDL_Rect tmp = {0,0,0,0};

	buttonincrement = new GuiClickableBtn( tmp, this, "" );
	buttondecrement = new GuiClickableBtn( tmp, this, "" );
	buttonmove = new GuiMoveableBtn( tmp, this, "" );

	buttonincrement->setBorderVisible( false );
	buttondecrement->setBorderVisible( false );
	buttonmove->setBorderVisible( false );

	buttonincrement->setCallbackData( this );
	buttondecrement->setCallbackData( this );
	buttonmove->setCallbackData( this );
	buttonincrement->setClickCallback( &onButton );
	buttondecrement->setClickCallback( &onButton );
	buttonmove->setPosChangeCallback( &onButton, this );
	buttonmove->setClickCallback( &onButton );

	max = 0;
	min = 0;
	tick = 1;
	pos = 0;
	SDL_Rect r = getRelWindow();
	dir = r.w < r.h ? vertikal : horizontal;

	arrange();
}

GuiScroller::~GuiScroller()
{
}

void GuiScroller::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("UseTheme");
		bThemeBtns = str == NULL || strToBool(str);
		str = el->Attribute("OnChange");
		if(str)	{
			cbChange = getFunctionByName(str);
		}
	}
	GuiPictureBg::onInit();
}

int GuiScroller::setScrollPos(int p)
{
	pos = p;
	if( pos > max )
		pos = max;
	if( pos < min )
		pos = min;
	updateitempos();
	return pos;
}

int GuiScroller::setMinMax(int mi, int ma)
{
	max = ma;
	min = mi;
	if(max < min)
		max = min;
	setScrollPos(pos);
	updateitemrange();
	return pos;
}

Uint32 GuiScroller::onMouseDown( SDL_MouseButtonEvent *ev )
{
	return 0;
}

Uint32 GuiScroller::onMouseUp( SDL_MouseButtonEvent *ev )
{
	return 0;
}

Uint32 GuiScroller::onResize()
{
	arrange();
	return GuiElement::onResize();
}

Uint32 GuiScroller::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiPictureBg::onThemeChanged( t );
	if( t == 0 )
		return r;

	const char *name;
	if( bThemeBtns )	{
		name = t->getString( ( string(getClassName()) + "->button-bmps-move" ).c_str() );
		if( name )
			buttonmove->setButtonBmps( SdlManager::the->getImagesByName( name ) );
		if( dir == vertikal )	{
			name = t->getString( ( string(getClassName()) + "->button-bmps-up" ).c_str() );
			if( name )
				buttonincrement->setButtonBmps( SdlManager::the->getImagesByName( name ) );
			name = t->getString( ( string(getClassName()) + "->button-bmps-down" ).c_str() );
			if( name )
				buttondecrement->setButtonBmps( SdlManager::the->getImagesByName( name ) );
			name = t->getString( ( string(getClassName()) + "->background-bmp-ud" ).c_str() );
			if( name )
				setBmp( SdlManager::the->getImageByName( name ) );
		}
		else	{
			name = t->getString( ( string(getClassName()) + "->button-bmps-right" ).c_str() );
			if( name )
				buttonincrement->setButtonBmps( SdlManager::the->getImagesByName( name ) );
			name = t->getString( ( string(getClassName()) + "->button-bmps-left" ).c_str() );
			if( name )
				buttondecrement->setButtonBmps( SdlManager::the->getImagesByName( name ) );
			name = t->getString( ( string(getClassName()) + "->background-bmp-rl" ).c_str() );
			if( name )
				setBmp( SdlManager::the->getImageByName( name ) );
		}
	}
	else	{
		if( dir == vertikal )	{
			buttonincrement->setButtonBmps( SdlManager::the->getImagesByName("up button") );
			buttondecrement->setButtonBmps( SdlManager::the->getImagesByName("down button") );
		}
		else	{
			buttonincrement->setButtonBmps( SdlManager::the->getImagesByName("right button") );
			buttondecrement->setButtonBmps( SdlManager::the->getImagesByName("left button") );
		}
		buttonmove->setButtonBmps( SdlManager::the->getImagesByName( "round button" ) );
	}

	return r;
}

void GuiScroller::arrange()
{
	Uint32 width, height;
	getWndSize( width, height );
	SDL_Rect rect;

	if( dir == vertikal )	{
		rect.w = width;
		rect.x = 0;
		rect.y = 0;
		rect.h = 20;
		buttonincrement->setRelWindow( rect );
		rect.y = height - 20;
		rect.h = 20;
		buttondecrement->setRelWindow( rect );
	}
	else	{
		rect.x = 0;
		rect.y = 0;
		rect.w = 20;
		rect.h = height;
		buttondecrement->setRelWindow( rect );
		rect.x = width - 20;
		rect.w = 20;
		buttonincrement->setRelWindow( rect );
	}
	updateitemrange();
}

void GuiScroller::updateitempos()
{
	Uint32 width, height;
	getWndSize( width, height );
	Uint32 x, y;

	if(dir == vertikal)	{
		int top = 30;
		int bottom = height - 30;
		x = width / 2;
		if(bottom == top || max == min)
			y = bottom;
		else
			y = -(((pos - min) * (bottom - top)) / (max - min) - bottom);
		buttonmove->moveRelCenterTo( x, y );
	}
	else	{
		int left = 30;
		int right = width - 30;
		y = height / 2;
		if(left == right || max == min)
			x = left;
		else
			x = -(((pos - min) * (left - right)) / (max - min) - left);
		buttonmove->moveRelCenterTo( x, y );
	}
}

void GuiScroller::updateitemrange()
{
	Uint32 width, height;
	getWndSize( width, height );
	Uint32 w, h;
	int xStart, yStart,
		xEnd, yEnd;
	
	if( dir == vertikal )	{
		w = width;
		h = 20;
		buttonmove->resizeTo( w, h );
		buttonmove->setDirection( GuiMoveableBtn::vertikal );
		xStart = xEnd = width / 2;
		yStart = 30;
		yEnd = height - 30;
		if( max == min )	{
			xEnd = xStart;
			yEnd = yStart;
		}
		buttonmove->setRange( xStart, yStart, xEnd, yEnd );
	}
	else	{
		w = 20;
		h = height;
		buttonmove->resizeTo( w, h );
		buttonmove->setDirection( GuiMoveableBtn::horizontal );
		yStart = yEnd = height / 2;
		xStart = 30;
		xEnd = width - 30;
		if( max == min )	{
			xEnd = xStart;
			yEnd = yStart;
		}
		buttonmove->setRange( xStart, yStart, xEnd, yEnd );
	}
	updateitempos();
}

void GuiScroller::increment()
{
	pos += tick;
	if(pos > max)
		pos = max;
	updateitempos();
	if(cbChange.func)
		(*cbChange.func)(this,cbChange.obj,NULL);
}

void GuiScroller::decrement()
{
	pos -= tick;
	if( pos < min )
		pos = min;
	updateitempos();
	if(cbChange.func)
		(*cbChange.func)(this,cbChange.obj,NULL);
}

void GuiScroller::updatefromitem()
{
	Uint32 width, height;
	getWndSize( width, height );

	if( dir == vertikal )	{
		int top = 30;
		int bottom = height - 30;
		Sint32 x, p;
		buttonmove->getRelCenterPoint( x, p );
		pos = ( ( p - bottom ) * ( max - min ) * 2 / ( top - bottom ) + 1 ) / 2 + min;
	}
	else	{
		int left = 30;
		int right = width - 30;
		Sint32 y, p;
		buttonmove->getRelCenterPoint( p, y );
		pos = ( ( p - left ) * ( max - min ) * 2 / ( right - left ) + 1 ) / 2 + min;
	}
	if( buttonmove->getState() != GuiDynamicButton::pressed )
		updateitempos();
	if(cbChange.func)
		(*cbChange.func)(this,cbChange.obj,NULL);
}

void GuiScroller::setDirection( direction d )
{
	dir = d;
	arrange();
	GuiScroller::onThemeChanged( ThemeManager::the ? ThemeManager::the->getActiveTheme() : 0 );
}

//////////////////////////////////////////////////////////////////////
// GuiScrollBg

GuiScrollBg::GuiScrollBg( const SDL_Rect &r, GuiElement *el, const char *name, int minw, int minh, SdlImage * bmp )
	: GuiElement( r, el, name )
{
	cb.func = NULL;
	nMinWidth = minw;
	nMinHeight = minh;
}

GuiScrollBg::GuiScrollBg( TiXmlElement *el, GuiElement *p ): GuiElement( el, p )
{
	cb.func = NULL;
	
	nMinWidth = getRelWindow().w;
	nMinHeight = getRelWindow().h;
}

void GuiScrollBg::init( SdlImage * bmp )
{
	SDL_Rect f = { 0, 0, nMinWidth, nMinHeight };
	bg = new GuiFlowWindow( f, this, "", 0, bmp );

	SDL_Rect r = getRelWindow(), rr = r;
	rr.x = rr.y = 0;
	f = rr;
	rr.x = rr.w - 20;
	rr.w = 20;
	scrolly = new GuiScroller( rr, this, "" );

	rr.y += rr.h - 20;
	rr.h = 20;
	gpbCorner = new GuiPictureBg( rr, this, "", 0, bmp );
	gpbCorner->toTop();

	f.y = f.h - 20;
	f.h = 20;
	scrollx = new GuiScroller( f, this, "" );
	scrollx->setCallbackData( this );
	scrollx->setChangeCallback( onScroll );
	scrolly->setCallbackData( this );
	scrolly->setChangeCallback( onScroll );
	scrollx->setMinMax( 0, MAX( 0, nMinWidth - r.w + rr.w ) );
	scrolly->setMinMax( 0, MAX( -rr.h, nMinHeight - r.h ) );
	scrollx->setDirection( GuiScroller::horizontal );
	scrolly->setDirection( GuiScroller::vertikal );
	if( scrollx->getMax() == scrollx->getMin() )
		scrollx->setVisible( false );
	else	{
		f = scrolly->getRelWindow();
		f.h -= 20;
		scrolly->setRelWindow( f );
		scrollx->setVisible( true );
	}
	if( scrolly->getMax() == scrolly->getMin() )
		scrolly->setVisible( false );
	else	{
		f = scrollx->getRelWindow();
		f.w -= 20;
		scrollx->setRelWindow( f );
		scrolly->setScrollPos( scrolly->getMin() );
		scrolly->setVisible( true );
	}
	bg->eFlyStyle = GuiFlowWindow::powsin;
	scrolly->setScrollPos( MAX( -rr.h, nMinHeight - r.h ) );
}

GuiScrollBg::~GuiScrollBg()
{

}

void GuiScrollBg::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("OnScroll");
		if(str)
			cb = getFunctionByName(str);
		
		unsigned long val;
		if(el->QueryULongAttribute("MinWith", &val) == TIXML_SUCCESS)
			nMinWidth = val;
		if(el->QueryULongAttribute("MinHeight", &val) == TIXML_SUCCESS)
			nMinHeight = val;
	}
	GuiElement::onInit();
}

void GuiScrollBg::onScroll( GuiScroller *sender )
{
	if( ( sender == scrollx )||( sender == scrolly ) )
	{
		int x = scrollx->getScrollPos(),
			y = scrolly->getScrollPos();
		SDL_Rect r = { -x, y-nMinHeight+getRelWindow().h, nMinWidth, nMinHeight };
		bg->setRelWindow( r );
		if( cb.func )
			(*cb.func)( this, cb.obj, NULL );
	}
}

Uint32 GuiScrollBg::onResize()
{
	arrange();
	return 0;
}

void GuiScrollBg::setMinSize( int w, int h )
{
	SDL_Rect r = getRelWindow();
	nMinWidth = MAX( w, r.w );
	nMinHeight = MAX( h, r.h );
	arrange();
}

void GuiScrollBg::arrange()
{
	SDL_Rect r = getRelWindow();
	r.x = r.y = 0;
	SDL_Rect f = { 0, 0, nMinWidth, nMinHeight };
	bg->setRelWindow( f );
	f = r;
	r.x = r.w - 20;
	r.w = 20;
	scrolly->setRelWindow( r );

	r = f;
	f.y = f.h - 20;
	f.h = 20;
	scrollx->setRelWindow( f );
	scrollx->setMinMax( 0, MAX( 0, nMinWidth - r.w ) );
	scrolly->setMinMax( -MAX( 0, nMinHeight - r.h ), 0 );
	scrollx->setDirection( GuiScroller::horizontal );
	scrolly->setDirection( GuiScroller::vertikal );
	if( scrollx->getMax() == scrollx->getMin() )
		scrollx->setVisible( false );
	else	{
		scrollx->tick = MAX( 1, ( scrollx->getMax() - scrollx->getMin() ) / 20 );
		f = scrolly->getRelWindow();
		f.h -= 20;
		scrolly->setRelWindow( f );
		scrollx->setVisible( true );
	}
	if( scrolly->getMax() == scrolly->getMin() )
		scrolly->setVisible( false );
	else	{
		scrolly->tick = MAX( 1, ( scrolly->getMax() - scrolly->getMin() ) / 20);
		f = scrollx->getRelWindow();
		f.w -= 20;
		scrollx->setRelWindow( f );
		scrolly->setVisible( true );
	}
	bg->eFlyStyle = GuiFlowWindow::powsin;
}

