/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2006 - 2007 by Tobias Nadler                                  */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <GL/gl.h>
#include "guisurface.h"
#include "sdlsurface.h"
#include "sdlmanager.h"
#include "tinyxml.h"

DECLARE_SLOT(new, GuiSurface, GuiSurface::newguiobject)

GuiSurface::GuiSurface( const SDL_Rect &r, GuiElement *p, const char *name ): GuiElement( r, p, name )
{
	nUnuseCounter = 0;
	if( SdlManager::the->isUsingOpenGL() )
	{
		static_data.nListID = glGenLists( 1 );
	}
	else
	{
		if(r.w > 0 && r.h > 0)	{
			static_data.surfDest = new SdlSurface(r.w, r.h);
		}
		else	{
			static_data.surfDest = NULL;
		}
	}
}

GuiSurface::GuiSurface( TiXmlElement *el, GuiElement *p ): GuiElement( el, p )
{
	nUnuseCounter = 0;
	if( SdlManager::the->isUsingOpenGL() )
	{
		static_data.nListID = glGenLists( 1 );
	}
	else
	{
		SDL_Rect r = getRelWindow();
		if(r.w > 0 && r.h > 0)	{
			static_data.surfDest = new SdlSurface(r.w, r.h);
		}
		else	{
			static_data.surfDest = NULL;
		}
	}
}

GuiSurface::~GuiSurface()
{
	if( SdlManager::the->isUsingOpenGL() )
	{
		glDeleteLists( static_data.nListID, 1 );
	}
}

void GuiSurface::onInit()
{
	TiXmlElement *xmlData = getXmlData();
	if( xmlData )	{
		nUnuseCounter = !xmlData->Attribute("Inactive") ? 0 : 1;
	}
	GuiElement::onInit();
}

// event-handling

Uint32 GuiSurface::onResize()
{
	if( SdlManager::the->isUsingOpenGL() )
	{
		setRedraw();
	}
	else
	{
		SDL_Rect r = getRelWindow();
		if(static_data.surfDest && static_data.surfDest->getWidth() <= r.w && static_data.surfDest->getHeight() <= r.h)
			return 0;
		if(static_data.surfDest)
			delete static_data.surfDest;
		static_data.surfDest = new SdlSurface(r.w, r.h);
		setRedraw();
	}
	return 0;
}

Uint32 GuiSurface::onFrame(SdlSurface *dest)
{
	if( nUnuseCounter || static_data.surfDest == NULL )
		return GuiElement::onFrame(dest);

	Uint32 result = GuiElement::onFrame( static_data.surfDest );
	
	dest->setClip( getClip() );
	SDL_Rect dst = getDrawWindow();
	dest->fastBlit( static_data.surfDest, dst.x, dst.y, NULL );
	return result;
}

Uint32 GuiSurface::onGLFrame()
{
	if( nUnuseCounter || static_data.nListID == 0 )
		return GuiElement::onGLFrame();

	if( getRedraw() || !hasSurface() )
	{
		glNewList( static_data.nListID, GL_COMPILE_AND_EXECUTE );
		Uint32 result = GuiElement::onGLFrame();
		glEndList();
		return result;
	}
	
	glCallList( static_data.nListID );
	return 0;
}

