/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "guiupdown.h"
#include <string>
#include <math.h>
#include "uimanager.h"
#include "thememanager.h"
#include "basicapp.h"
#include "functions.h"

DECLARE_SLOT(new, GuiUpDown, GuiUpDown::newguiobject)

GuiUpDown::GuiUpDown( const SDL_Rect &r, GuiElement *el, const char *name, int val, bool usetheme ): GuiPictureBg( r, el, name )
{
	bThemeBtns = usetheme;
	value = val;
	increment = 1;
	min = MIN_SINT32;
	max = MAX_SINT32;
	cb.func = 0;

	SDL_Rect rr = r;
	rr.x = rr.y = 0;
	rr.w -= 25;
	text = new GuiTextInput( rr, this, "", intToStr( value, 10 ), true );
	text->setMaxlen( 10 );
	text->setAlign( SdlFont::baseline, SdlFont::right );
	text->setCbData( this );
	text->setCallback( onText );
	rr.x += rr.w;
	rr.w = 25;
	rr.h = r.h / 2;

	up = new GuiClickableBtn( rr, this, "", 0, SdlManager::the->getImagesByName( "up button" ) );
	rr.y += rr.h;
	down = new GuiClickableBtn( rr, this, "", 0, SdlManager::the->getImagesByName( "down button" ) );

	up->setClickCallback( onUdBtn );
	up->setCallbackData( this );
	down->setClickCallback( onUdBtn );
	down->setCallbackData( this );

	GuiUpDown::onThemeChanged( ThemeManager::the ? ThemeManager::the->getActiveTheme() : 0 );
}

GuiUpDown::GuiUpDown( TiXmlElement *el, GuiElement *p ): GuiPictureBg( el, p )
{
	bThemeBtns = true;
	value = 0;
	increment = 1;
	min = -999999999;
	max = 999999999;
	cb.func = 0;

	SDL_Rect r, rr = r = getRelWindow();
	rr.x = rr.y = 0;
	rr.w -= 25;
	text = new GuiTextInput( rr, this, "", intToStr( value, 10 ), true );
	text->setMaxlen( 10 );
	text->setAlign( SdlFont::baseline, SdlFont::right );
	text->setCbData( this );
	text->setCallback( onText );
	rr.x += rr.w;
	rr.w = 25;
	rr.h = r.h / 2;

	up = new GuiClickableBtn( rr, this, "", 0, SdlManager::the->getImagesByName( "up button" ) );
	rr.y += rr.h;
	down = new GuiClickableBtn( rr, this, "", 0, SdlManager::the->getImagesByName( "down button" ) );

	up->setClickCallback( onUdBtn );
	up->setCallbackData( this );
	down->setClickCallback( onUdBtn );
	down->setCallbackData( this );
}

GuiUpDown::~GuiUpDown()
{

}

void GuiUpDown::onInit()
{
	TiXmlElement *el = getXmlData();
	if(el)	{
		const char *str = el->Attribute("UseTheme");
		if(str)
			bThemeBtns = strToBool(str);
		str = el->Attribute("Value");
		if(str)
			value = atoi(str);
		str = el->Attribute("MinValue");
		if(str)
			min = atoi(str);
		str = el->Attribute("MaxValue");
		if(str)
			max = atoi(str);
		setMinMax(min, max);
		str = el->Attribute("OnChange");
		if(str)
			cb = getFunctionByName(str);
	}
	GuiPictureBg::onInit();
}

void GuiUpDown::onButton( GuiClickableBtn *button )
{
	value = atoi( text->getText() );
	if( button == up )
	{
		*this += increment;
		UiManager::the->setFocus( ftKeyboard, text );
	}
	else if( button == down )
	{
		*this -= increment;
		UiManager::the->setFocus( ftKeyboard, text );
	}
	else
		return;

	if( cb.func )
		(*cb.func)( this, cb.obj, NULL );
}

Uint32 GuiUpDown::onThemeChanged( UiTheme *t )
{
	Uint32 r = GuiPictureBg::onThemeChanged( t );
	if( t == 0 )
		return r;

	if( bThemeBtns )	{
		const char *name, *name2;
		name = t->getString( ( string(getClassName()) + "->button-bmps-inc" ).c_str() );
		if( name )
			up->setButtonBmps( SdlManager::the->getImagesByName( name ) );
		name = t->getString( ( string(getClassName()) + "->button-bmps-dec" ).c_str() );
		if( name )
			down->setButtonBmps( SdlManager::the->getImagesByName( name ) );

		name = t->getString( ( string(getClassName()) + "->button-sound-up-inc" ).c_str() );
		name2 = t->getString( ( string(getClassName()) + "->button-sound-down-inc" ).c_str() );
		up->setSounds( name ? SdlManager::the->getAudioByName( name ) : 0,
					name2 ? SdlManager::the->getAudioByName( name2 ) : 0 );

		name = t->getString( ( string(getClassName()) + "->button-sound-up-dec" ).c_str() );
		name2 = t->getString( ( string(getClassName()) + "->button-sound-down-dec" ).c_str() );
		down->setSounds( name ? SdlManager::the->getAudioByName( name ) : 0,
					name2 ? SdlManager::the->getAudioByName( name2 ) : 0 );
		
		up->setBorderVisible( t->getBool( ( string(getClassName()) + "->showborder-inc" ).c_str() ) );
		down->setBorderVisible( t->getBool( ( string(getClassName()) + "->showborder-dec" ).c_str() ) );
	}
	return r;
}

int GuiUpDown::getValue()
{
	const char *t = text->getText();// strlen(t) is always <= 10
	if(*t == '-')// t cannot be too long, if negative
		value = atoi( t );
	else if(strlen(t) > 9)// check, if t too long
		value = 999999999;
	else value = atoi( t );
	refresh();
	return value;
}

void GuiUpDown::arrange()
{
	SDL_Rect r = getRelWindow();
	r.x = r.y = 0;
	r.w -= 25;
	text->setRelWindow( r );
	r.x += r.w;
	r.w = 25;
	r.h /= 2;
	up->setRelWindow( r );
	r.y = r.h;
	down->setRelWindow( r );
}

Uint32 GuiUpDown::onResize()
{
	arrange();
	return 0;
}

void GuiUpDown::setMinMax(int mi, int ma)
{
	max = ma;
	min = mi;
	if(max < min)
		max = min;
	if(max > 0 && !(min < 0 && -min*10 > max))
		text->setMaxlen((int)(log(max)/log(10) + 1.0));
	else if(min < 0)
		text->setMaxlen((int)(log(-min)/log(10) + 2.0));
	else text->setMaxlen(1);// max == min == 0
	refresh();
}

void GuiUpDown::setValue(int v)
{
	value = v > max ? max : (v < min ? min : v);
	char buf[33];
	text->setText( intToStr( value, buf ) );
}

int GuiUpDown::operator ++()
{
	value++;
	refresh();
	return value;
}

int GuiUpDown::operator +=(int i)
{
	value += i;
	refresh();
	return value;
}

int GuiUpDown::operator --()
{
	value--;
	refresh();
	return value;
}

int GuiUpDown::operator -=(int i)
{
	value -= i;
	refresh();
	return value;
}

Uint32 GuiUpDown::onRelease()
{
	value = atoi( text->getText() );
	refresh();
	if( cb.func )
		(*cb.func)( this, cb.obj, NULL );
	return 0;
}

void GuiUpDown::refresh()
{
	if( value < min )
		value = min;
	else if( value > max )
		value = max;
	char ch[33];
	text->setText( intToStr( value, ch ) );
}
