/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "netthread.h"
#ifndef WEB_CLIENT
#include "webserver.h"
#include "webpage.h"
#endif
#include "netsocket.h"
#include "netpeermanager.h"
#include "functions.h"

NetThread *NetThread::the = 0;
NetThread *NetThread::getThe()
{
	return the;
}


#ifdef WEB_CLIENT
NetThread::NetThread( const char *name, BasicObject *p, bool debug )
	: BasicThread( name, p, debug ? "net-debug.log" : 0 )
{
	the = this;
	logBegin("Net-thread created.");
}
#else
NetThread::NetThread( const char *name, BasicObject *p, bool server, bool debug )
	: BasicThread( name, p, debug ? "net-debug.log" : 0 )
{
	bServer = server;
	the = this;
	logBegin("Net-thread created.");
}
#endif

NetThread::NetThread( TiXmlElement *el, BasicObject *p )
	: BasicThread( el, p )
{
	the = this;
	logBegin("Net-thread created.");
}

NetThread::~NetThread()
{
	stop( 100 );
	the = 0;
	logEnd("Net-thread destructed.");
}

void NetThread::connectSocket( NetSocket *s, const IPaddress &ip, BasicObject::Callback c, BasicObject *obj )
{
	BasicObject::FunctionData fd(c, obj);
	lToConnect.push_back( DATA( s, ip, fd ) );
}

void NetThread::onInit()
{
	if( xmlData )	{
#ifndef WEB_CLIENT
		const char *s = xmlData->Attribute("Server");
		bServer = s && strToBool(s);
#endif
		if( !xmlData->Attribute("StartOnInit") || strToBool(xmlData->Attribute("StartOnInit")) )
			start();
	}
	else
		start();
}

void NetThread::onDestroy()
{
	if( xmlData )	{
		if( xmlData->Attribute("StartOnInit") && strToBool(xmlData->Attribute("StartOnInit")) )
			stop(100);
	}
}

int NetThread::onRun()
{
	BasicThread::onRun();
	return 0;
}

void NetThread::onCycle()
{
	lock( 10 );
	NetBasicManager::the->checkPeers();
	NetBasicManager::the->checkServer();// also needed as peer (not only as server)
	while( lToConnect.size() )	{
		bool r = lToConnect.front().s->connect(lToConnect.front().ip);
		if( lToConnect.front().cb.func )
			(*lToConnect.front().cb.func)( lToConnect.front().s, lToConnect.front().cb.obj, &r );
		lToConnect.pop_front();
	}
#ifndef WEB_CLIENT
	if( bServer && WebServer::the )	{
		WebServer::the->poll();
		if( (numCycles() & 0x7F) == 0 )
			WebServer::the->closeOutTimedThreads();
	}
#endif
	if( (numCycles() & 0x7F) == 0 )
		NetBasicManager::the->deleteDisconnected();
	unlock();
	SDL_Delay( 1 );
}
