/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include <fstream>
#include "webfilepage.h"
#include "webserver.h"
#include "basicapp.h"

#define WEB_PACKET_SIZE	1400

using namespace std;

WebFilePage::WebFilePage( const char *uri, const char *fn, const char *type ): WebPage( uri )
{
	if( fn )
		strFilename = BasicApp::the->makeDataPath( fn );
	else
		strFilename = BasicApp::the->makeDataPath( (uri + 1) );
	if( type )
		strcpy( chContentType, type );
	else	{
		const char *chEnding = strrchr( uri, '.' );
		if( chEnding == 0 || strstr( chEnding + 1, "htm" ) != NULL || strcmp( chEnding + 1, "txt" ) == 0 )
			strcpy( chContentType, "text/html" );
		else if( strcmp( chEnding + 1, "ico" ) == 0 )
			strcpy( chContentType, "image/x-icon" );
		else if( strcmp( chEnding + 1, "png" ) == 0 )
			strcpy( chContentType, "image/png" );
		else if( strcmp( chEnding + 1, "jpg" ) == 0 || strcmp( chEnding + 1, "jpeg" ) == 0 )
			strcpy( chContentType, "image/jpeg" );
		else if( strcmp( chEnding + 1, "gif" ) == 0 )
			strcpy( chContentType, "image/gif" );
		else if( strcmp( chEnding + 1, "pdf" ) == 0 )
			strcpy( chContentType, "application/pdf" );
		else if( strcmp( chEnding + 1, "zip" ) == 0 )
			strcpy( chContentType, "application/zip" );
		else //if( strcmp( chEnding + 1, "exe" ) == 0 || strcmp( chEnding + 1, "tar" ) == 0 ||
				//strcmp( chEnding + 1, "rar" ) == 0 || strcmp( chEnding + 1, "gz" ) == 0 )
			strcpy( chContentType, "application/octet-stream" );
		//else
		//	strcpy( chContentType, "text/html" );
	}
}

WebFilePage::~WebFilePage()
{
}

int WebFilePage::send( WebSrvClient *client, Uint32 &responsecode )
{
	ifstream file( strFilename.c_str(), ios::in | ios::binary );

	if( !file.is_open() )	{
		WebSrv_SendHTTPResponse( client, 400 );// bad request
		responsecode = 400;
		return 1;
	}
	WebSrv_SendHTTPResponse( client, 200 );// ok
	responsecode = 200;
	
	int nBytes = 0;
	char tmp[WEB_PACKET_SIZE];
	strcpy( tmp, "Content-Type: " );
	strcat( tmp, chContentType );
	strcat( tmp, "\r\n\r\n" );
	nBytes += strlen( tmp );
	WebSrv_ClientSendString( client, tmp );

	do	{
		int i;
		for( i = 0; i < WEB_PACKET_SIZE && file.good(); ++i )
			tmp[i] = file.get();
//		file.read( tmp, 1024 );// todo <-- find out how much was read
		nBytes += SDLNet_TCP_Send( client->sock, tmp, i-1 );
	} while( file.good() );
	file.close();
	
	return nBytes;
}

