/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "webstatisticspage.h"
#include <string>
#include "netpeermanager.h"
#include "functions.h"

WebStatisticsPage::WebStatisticsPage( const char *uri ): WebPage( uri )
{
	chTitle = chHeading = chEnding = NULL;
	wTableTitles = 50;
	wTable = 90;
	wBorder = 1;
	colorTable = 0xEEEEFF;
}

WebStatisticsPage::~WebStatisticsPage()
{
	if( chTitle )
		delete[] chTitle;
	if( chHeading )
		delete[] chHeading;
	if( chEnding )
		delete[] chEnding;
}

WebStatisticsPage::ItemIter WebStatisticsPage::addItem( const char *title, Callback c, void *cb_data )
{
	TABLE_ROW r;
	r.chTitle = new char[strlen(title)+1];
	strcpy( r.chTitle, title );
	r.cb = c;
	r.cbData = cb_data;
	return lItems.insert( lItems.end(), r );
}

void WebStatisticsPage::removeItem( ItemIter i )
{
	lItems.erase( i );
}

void WebStatisticsPage::setTitle( const char *t )
{
	if( chTitle )
		delete[] chTitle;
	if( t == NULL )	{
		chTitle = NULL;
		return;
	}
	chTitle = new char[strlen(t)+1];
	strcpy( chTitle, t );
}

void WebStatisticsPage::setHeading( const char *t )
{
	if( chHeading )
		delete[] chHeading;
	if( t == NULL )	{
		chHeading = NULL;
		return;
	}
	chHeading = new char[strlen(t)+1];
	strcpy( chHeading, t );
}

void WebStatisticsPage::setEnding( const char *t )
{
	if( chEnding )
		delete[] chEnding;
	if( t == NULL )	{
		chEnding = NULL;
		return;
	}
	chEnding = new char[strlen(t)+1];
	strcpy( chEnding, t );
}


int WebStatisticsPage::send( WebSrvClient *client, Uint32 &responsecode )
{
	WebSrv_SendHTTPResponse( client, 200 );// ok
	responsecode = 200;
	
	NetPeerManager *pm = (NetPeerManager*)NetBasicManager::the;
	pm->requestPings();
	int nBytes = 0;
	char buf[1024], bufnum[33];
	char *tmp;
	nBytes += sendString( client, "Content-Type: text/html\r\n\r\n"
				"<html><head><title>" );
	nBytes += sendString( client, chTitle );
	nBytes += sendString( client, "</title></head><body>\r\n<h1 align=\"center\">" );
	nBytes += sendString( client, chHeading );
	sprintf( buf, "</h1>\r\n<table width=\"%d%%\" border=\"%d\" bgcolor=\"#%X\" align=\"center\">\r\n", wTable, wBorder, colorTable );
	nBytes += sendString( client, buf );
	strcpy( buf, "<tr><td width=\"" );
	strcat( buf, intToStr( wTableTitles, bufnum ) );
	strcat( buf, "%\">" );
	for( ItemIter i = lItems.begin(); i != lItems.end(); ++i )	{
		if( (*i).cb == NULL )
			continue;
		tmp = (*(*i).cb)( this, (*i).cbData, (*i).chTitle, client );
		if( tmp == NULL )
			continue;
		nBytes += sendString( client, buf );
		nBytes += sendString( client, (*i).chTitle );
		nBytes += sendString( client, "</td><td>" );
		nBytes += sendString( client, tmp );
		nBytes += sendString( client, "</td></tr>\r\n" );
		delete[] tmp;
	}

	nBytes += sendString( client, "</table>\r\n" );
	nBytes += sendString( client, chEnding );
	nBytes += sendString( client, "\r\n</body></html>" );
	
	return nBytes;
}

