/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "webthread.h"
#include "SDL/SDL.h"
#include "webserver.h"
#include "webpage.h"
#include "functions.h"
#include "basicapp.h"

Uint32 WebThread::tIdleToExit = 30000;

WebThread::WebThread( const char *name, BasicObject *p, const char * fndebug ): BasicThread( name, p, fndebug )
{
	tLastAction = SDL_GetTicks();
	WebServer::the->lThreads.push_back( this );
}

WebThread::~WebThread()
{
	WebServer::the->lThreads.remove( this );
}

int WebThread::onRun()
{
	tLastAction = SDL_GetTicks();

	client = WebSrv_Accept( WebServer::the->webServer );
	if( !client )	{
		WebServer::the->logLine( "Invalid connection request." );
		return 0;
	}

	const char *chKind;
	switch( client->url.encoding )	{
		case WEBSRV_URL:
			chKind = client->post_length && client->post ? "POST " : "GET " ;
			break;
		case WEBSRV_MULTIPART:
			chKind = "MULTIPART ";
			break;
		case WEBSRV_INVALID:
		default:
			chKind = "INVALID ";
			break;
	}

	char *request = new char[ strlen(client->url.path) + strlen(client->url.file) + 2 ];
	if( client->url.path[0] == '\0' )
		strcpy( request, "/" );
	else	{
		strcpy( request, client->url.path );
		strcat( request, "/" );
	}
	strcat( request, client->url.file );

	int nBytes = 0;
	Uint32 responsecode = 0;
//	BasicApp::the->logLine( "---" );
//	BasicApp::the->logLine( request );
	for( list<WebPage*>::iterator page = WebServer::the->lPages.begin(); page != WebServer::the->lPages.end(); ++page )	{
		if( (*page)->getURI() && strcmp( (*page)->getURI(), request ) == 0 )	{
			if( ( nBytes = (*page)->send( client, responsecode ) ) > 0 )
				break;
		}
/*		else	{//just for debugging
			BasicApp::the->logLine( (*page)->getURI() );
		}*/
	}
	delete[] request;

	char buf[33];
	if( nBytes == 0 )	{
		WebSrv_SendHTTPResponse( client, 400 );// bad request
		char *tmp = "Content-Type: text/html\r\n\r\n";
		WebSrv_ClientSendString( client, tmp );
		tmp = "<html><body>\r\n<h2>Page not availible</h2>\r\n";
		WebSrv_ClientSendString( client, tmp );
		tmp = "<p>This page does not exist.</p>\r\n</body></html>\r\n";
		WebSrv_ClientSendString( client, tmp );
		tmp = new char[strlen(client->url.url) + 80];
		IPToStr( *SDLNet_TCP_GetPeerAddress(client->sock), tmp );
		strcat( tmp, " - - [" );
		strcat( tmp, intToStr( tLastAction, buf ) );
		strcat( tmp, "] \"" );
		strcat( tmp, chKind );
		strcat( tmp, client->url.url );
		strcat( tmp, "\" - 400 - 0 - " );
		strcat( tmp, intToStr( SDL_GetTicks() - tLastAction, buf ) );
		WebServer::the->logLine( tmp );
		delete[] tmp;
	}
	else	{
		char *tmp = new char[strlen(client->url.url) + 80];
		sprintf(tmp, "%s - - [%d] \"%s%s\" - %d - %d - %d",
				IPToStr( *SDLNet_TCP_GetPeerAddress(client->sock), buf ),
				tLastAction, chKind, client->url.url, responsecode, nBytes,
				SDL_GetTicks() - tLastAction);
		WebServer::the->logLine( tmp );
		delete[] tmp;
	}

	tLastAction = SDL_GetTicks();
	WebSrv_CloseClient( WebServer::the->webServer, client );
	client = NULL;

	return 0;
}
