/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "functions.h"
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifndef __windows__
#include <unistd.h>
#include <pwd.h>
#include <ifaddrs.h>
#include <netdb.h>
#endif

const char *intToStr( int i, int base )
{
	static char chStr[33];// = "00000000000000000000000000000000";
	return intToStr( i, chStr, base );
}

char *intToStr( int i, char *buf, int base )
{
	static const char *chNumbers = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	if( i == 0 )	{
		buf[31] = '0';
		buf[32] = '\0';
		return &buf[31];
	}

	if( base < 2 )
		base = 2;
	else
		if( base > 36 )
			base = 36;

	bool sign = i < 0;
	if( sign )	{
		//if( (i & 0x80000000) != 0 )
		//	return uintToStr( i, buf, base );
		i = -i;
	}
	int n = 31;
	for( ; i != 0; --n )	{
		buf[n] = chNumbers[i % base];
		i /= base;
	}
	if( sign )
		buf[n] = '-';
	else
		++n;
	buf[32] = '\0';
	return buf + n;
}

char *uintToStr( unsigned int i, char *buf, unsigned int base )
{
	static const char *chNumbers = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	if( i == 0 )	{
		buf[31] = '0';
		buf[32] = '\0';
		return &buf[31];
	}

	if( base < 2 )
		base = 2;
	else
		if( base > 36 )
			base = 36;

	int n = 31;
	for( ; i != 0; --n )	{
		buf[n] = chNumbers[i % base];
		i /= base;
	}
	++n;
	buf[32] = '\0';
	return buf + n;
}


Uint32 xStrToUL( const char *str, bool *err )
{
	while( *str == ' ' )
		++str;
	int base = 10;
	if( *str == '0' )	{
		++str;
		base = 8;
	}
	switch( *str )
	{
	case '#':
	case 'h':
	case '$':
	case 'x':
		base = 16;
		++str;
		break;
	case 'b':
		base = 2;
		++str;
		break;
	case 'd':
		base = 10;
		++str;
		break;
	case 'o':
		base = 8;
		++str;
		break;
	case 'q':
		base = 4;
		++str;
		break;
	};

	if(err)	{
		char *p;
		unsigned long r = strtoul( str, &p, base );
		*err = p == str;
		return r;
	}
	else
		return strtoul( str, 0, base );
}

SDL_Rect strToRect( const char *str )
/// \todo verify ',' as separator
{// example: 10,50,30,30 => result = {10,50,30,30};
	const char *chVals[4];
	chVals[0] = str;
	int i, n = 0;
	for( i = 0; str[i] != '\0' && str[i] != '\n'; ++i )
		switch( str[i] )	{
		case ' ':
		case '\t':
//			str[i] = '\0';
			chVals[n] = &str[i+1];
			break;
		case ',':
		case ';':
//			str[i] = '\0';
			chVals[++n] = &str[i+1];

			if( n == 3 )
				break;
		}

	SDL_Rect r = { atoi(chVals[0]), atoi(chVals[1]), atoi(chVals[2]), atoi(chVals[3])};

	return r;
}

bool strToBool( const char *s )
{
	if( strcmp( s, "1" ) == 0 || strcmp( s, "+" ) == 0 )
		return true;
	char tmp[16];
	int i;
	for( i = 0; i < 15 && s[i]; ++i )
		tmp[i] = toupper( s[i] );
	tmp[i] = '\0';
	if( strcmp( tmp, "TRUE" ) == 0 || strcmp( tmp, "YES" ) == 0 || strcmp( tmp, "Y" ) == 0 ||
		strcmp( tmp, "OK"   ) == 0 || strcmp( tmp, "ON"  ) == 0 )
		return true;

	return false;
}

Uint32 strToIP( const char *str )
{
	Uint32 result;
	const char *tmp = strchr( str, '.' );
	if( tmp == 0 )	{
		result = xStrToUL( str );
	}
	else	{
		char chPart[4];
		strncpy( chPart, str, 3 );
		chPart[ tmp-str ] = '\0';
		result = atoi( chPart ) << 24;
		++tmp;

		const char *tmp2 = strchr( tmp, '.' );
		if( tmp2 == 0 )
			goto STRTOIP_END;
		strncpy( chPart, tmp, 3 );
		chPart[ tmp2-tmp ] = '\0';
		result |= atoi( chPart ) << 16;

		tmp = tmp2+1;
		tmp2 = strchr( tmp, '.' );
		if( tmp2 == 0 )
			goto STRTOIP_END;
		strncpy( chPart, tmp, 3 );
		chPart[ tmp2-tmp ] = '\0';
		result |= atoi( chPart ) << 8;

//		if( (tmp = strchr( tmp, ':' )) != NULL )
//			*tmp = '\0';
		result |= atoi( tmp2+1 );
	}
STRTOIP_END:
	return SDLNet_Read32( &result );
}

IPaddress strToIPaddress( const char *str )
{
	IPaddress result;
	result.host = strToIP( str );
	const char *tmp;
	if( (tmp = strchr( str, ':' )) == NULL )
		result.port = 0;
	else	{
		++tmp;
		Uint16 i = atoi( tmp );
		result.port = SDLNet_Read16( &i );
	}
	return result;
}


SDLKey strToKey( const char *str )
{
static const char * keys8_32[] = {"backspace","tab","","","clear","return","","", "","","","pause","","","","", "","","","esc","","","",""};
static const char * keys_kp[] = {"period","/","*","-","+","enter","="};
static const char * keys_move[] = {"up","down","right","left","insert","home","end","pageup","pagedown"};
static const char * keys_misc[] = {"numlock","capslock","scrollock", "rshift","lshift","rctrl","lctrl","ralt","lalt", "rmeta","lmeta","lsuper","rsuper", "mode","compose", "help","print","sysreq","break","menu","power","euro","undo"};
	if(!str[1])
		return (SDLKey)*str;
	int n;
	for(n = 0; n < 24; ++n)
		if(strcmp(keys8_32[n], str) == 0)
			return (SDLKey)(n + 8);
	if(strcmp(str, "delete") == 0)
		return SDLK_DELETE;
	if(strcmp(str, "quote-dbl") == 0)
		return SDLK_QUOTEDBL;
	if(strcmp(str, "colon") == 0)
		return SDLK_COLON;
	if(*str == 'F')	{
		int i = atoi(str+1);
		if(i > 0 && i <= 15)
			return (SDLKey)(SDLK_F1-1 + i);
	}
	if(strncmp(str, "KP", 2) == 0)	{
		if(!str[3])	{
			if(str[2] >= '0' && str[2] <= '9')
				return (SDLKey)(256 + str[2] - '0');
		}
		for(n = 0; n < 7; ++n)
			if(strcmp(keys_kp[n], str+2) == 0)
				return (SDLKey)(n + SDLK_KP_PERIOD);
	}
	for(n = 0; n < 9; ++n)
		if(strcmp(keys_move[n], str) == 0)
			return (SDLKey)(n + SDLK_UP);
	
	for(n = 0; n < 23; ++n)
		if(strcmp(keys_misc[n], str) == 0)
			return (SDLKey)(n + SDLK_NUMLOCK);
	
	return SDLK_UNKNOWN;
}

SDLMod strToMod( const char *str )
{
	static const char * mod_strings[] = {"lsh","rsh","lct","rct","lal","ral","lme","rme", "num","cap","mod"};
	static const char * mod2_strings[] = {"shi","ctr","alt","met"};
	static const int mod2_value[] = {KMOD_SHIFT, KMOD_CTRL, KMOD_ALT, KMOD_META};
	int result = 0;
	while(*str)	{
		while(*str == ' ') str++;
		int n;
		for(n = 0; n < 4; ++n)
			if(strncmp(mod2_strings[n], str, 3) == 0)	{
				result |= mod2_value[n];
				goto AFTER_FOR;
			}
		for(n = 0; n < 2; ++n)
			if(strncmp(mod_strings[n], str, 3) == 0)	{
				result |= 1 << n;
				goto AFTER_FOR;
			}
		for(; n < 11; ++n)
			if(strncmp(mod_strings[n], str, 3) == 0)	{
				result |= 0x10 << n;
				break;
			}
AFTER_FOR:
		while(*++str && *str != '+');
	}
	return (SDLMod)result;
}


int byteToStr( Uint8 b, char *buf )
{
	int s = 0;
	if( b >= 100 )	{
		buf[0] = '0' + b/100;
		b %= 100;
		++s;
	}
	if( b >= 10 )	{
		buf[s] = '0' + b/10;
		b %= 10;
		++s;
	}
	buf[s] = '0' + b;
	buf[++s] = '\0';
	return s;
}

const char * IPToStr( Uint32 ip, char *buf )
{
	ip = SDLNet_Read32( &ip );
	int l = 0;
	for( int i = 3; i >= 0; --i )	{
		l += byteToStr( (Uint8)(ip >> i*8), buf + l );
		buf[l] = '.';
		++l;
	}
	buf[l-1] = '\0';
	return buf;
}

const char * IPToStr( const IPaddress &ip, char *buf )
{
	IPToStr( ip.host, buf );
	Uint16 port = SDLNet_Read16( &ip.port );
	char * pos = buf + strlen( buf );
	*pos = ':';
	bool b = false;
	for( Uint16 i = 10000; i > 0; i /= 10 )	{
		if( !b && (port / i || i == 1) )
			b = true;
		if( b )
			*++pos = '0' + port / i % 10;
	}
	*++pos = '\0';
	return buf;
}

SDL_Color ULToColor( Uint32 v )
{// better: convert with rgb-masks
	SDL_Color r;
	r.r = (Uint8)(v >> 16);
	r.g = (Uint8)(v >> 8);
	r.b = (Uint8)v;
	r.unused = 0;
	return r;
}

const char * getUserDirectory()
{
#ifdef _WINDOWS
	return getenv( "USERPROFILE" );
#else
	return getpwuid( getuid() )->pw_dir;
#endif
}

std::string makeEnvStr(const char *str)
{
	std::string s = str;
	const char *pnow = str, *p, *pend;
	int delta = 0;
	while(1)	{
		p = strstr(pnow, "$(");
		if(!p)
			break;
		pend = strchr(p, ')');
		if(!pend)	{
			pnow = p+2;
			continue;
		}
		pnow = pend+1;
		
		char *env = new char[pend-p-1];
		memcpy(env, p+2, pend - p - 2);
		env[pend - p - 2] = '\0';
		s.erase(p - str + delta, pend - p+1);
		const char *tmp = getenv(env);
		if(tmp)	{
			s.insert(p - str + delta, tmp);
			delta += strlen(tmp);
		}
		delta -= pend - p + 1;
	}
	return s;
}

std::string makeEnvPath(const char *str)
{
	if(*str != '~' || str[1] != '/')
		return makeEnvStr(str);
	std::string s = makeEnvStr(str);
	s.erase(0,2);
	s.insert(0, "/");
	s.insert(0, getUserDirectory());
	return s;
}

// -- the following two methods are from www.linuxquestions.org --
//
//  Case-sensitive pattern match
//
bool patMatch (const char* pattern, const char* string)
{
	switch (pattern[0])
	{
		case '\0':
			return !string[0];

		case '*':
			return patMatch(pattern+1, string) || string[0] && patMatch(pattern, string+1);

		case '?':
			return string[0] && patMatch(pattern+1, string+1);


		default:
			return (pattern[0] == string[0]) && patMatch(pattern+1, string+1);
	}
}

//
//  Case-insensitive pattern match
//
bool patiMatch (const char* pattern, const char* string)
{
	switch (pattern[0])
	{
		case '\0':
			return !string[0];

		case '*' :
			return patiMatch(pattern+1, string) || string[0] && patiMatch(pattern, string+1);

		case '?' :
			return string[0] && patiMatch(pattern+1, string+1);

		default  :
			return (toupper(pattern[0]) == toupper(string[0])) && patiMatch(pattern+1, string+1);
	}
}
// -- the previous two methods are from www.linuxquestions.org --

#ifndef __windows__
// thanks to http://www.hungry.com/~alves/local-ip-in-C.html for this method
char *getLocalIP( bool bIPv4 )
{
	static const uint szBuf = 64;
	char *strBuf = new char[szBuf];
	char strIP[szBuf];
	
	struct ifaddrs *pifaCurr = NULL, *pifaFirst = NULL;

	if( getifaddrs( &pifaFirst ) < 0 )
	{
		//perror ("getifaddrs");
		return NULL;
	}

	for( pifaCurr = pifaFirst; pifaCurr; pifaCurr = pifaCurr->ifa_next )
	{
		socklen_t szAddr;

		if( pifaCurr->ifa_addr->sa_family == AF_INET )
			szAddr = sizeof( struct sockaddr_in );
		else if( pifaCurr->ifa_addr->sa_family == AF_INET6 )
			szAddr = sizeof( struct sockaddr_in6 );
		else
			continue;
		
		if( getnameinfo( pifaCurr->ifa_addr, szAddr, strIP, szBuf, NULL, 0,
						 NI_NUMERICHOST ) < 0 )
		{
			//perror( "getnameinfo" );
			continue;
		}
		if( strstr( strIP, "127.0.0.1" ) == NULL )	{
			// take non-local address
			strcpy( strBuf, strIP );
			// if IPv4 take this address
			if( pifaCurr->ifa_addr->sa_family == AF_INET && bIPv4 )
				break;
			else if( pifaCurr->ifa_addr->sa_family == AF_INET6 && !bIPv4 )
				break;
		}
	}

	freeifaddrs( pifaFirst );

	return strBuf;
}
#endif

