/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "SDL/SDL.h"
#include "rectfunctions.h"

bool pointInRect( const SDL_Rect &r, int x, int y )
{
	if( ( r.x > x ) || ( r.x+r.w < x ) || ( r.y > y ) || ( r.y+r.h < y ) ) return false;
	return true;
}
bool pointInRect( const FloatRect &r, float x, float y )
{
	if( ( r.x > x ) || ( r.x+r.w < x ) || ( r.y > y ) || ( r.y+r.h < y ) ) return false;
	return true;
}

void setRectEmpty( SDL_Rect &r )		{	r.x = r.y = -1;	r.w = r.h = 0;	}
void setRectEmpty( FloatRect &r )		{	r.w = r.h = 0.f;	}

bool isRectEmpty( const SDL_Rect &r )	{ return r.w == 0   && r.x == -1; }
bool isRectEmpty( const FloatRect &r )	{ return r.w == 0.f || r.h == 0.f; }

void offsetRect( SDL_Rect &r, int dx, int dy )		{	r.x += dx;	r.y += dy;	}
void offsetRect( FloatRect &r, float dx, float dy )	{	r.x += dx;	r.y += dy;	}

void setRect( SDL_Rect &r, int x, int y, int w, int h )
{
	r.x = x;	r.y = y;	r.w = w;	r.h = h;
}

void setRect( FloatRect &r, float x, float y, float w, float h )
{
	r.x = x;	r.y = y;	r.w = w;	r.h = h;
}


void inflateRect( SDL_Rect &r, int dx, int dy )
{
	if( dx*2 < -r.w )	{
		r.x += r.w / 2;
		r.w &= 0xFFFE;// make even width
	}
	else	{
		r.x -= dx;
		r.w += dx * 2;
	}
	if( dy*2 < -r.h )	{
		r.y += r.h / 2;
		r.h &= 0xFFFE;
	}
	else	{
		r.y -= dy;
		r.h += dy * 2;
	}
}

void inflateRect( FloatRect &r, float dx, float dy )
{
	if( dx*2.f < -r.w )	{
		r.x += r.w / 2.f;
	}
	else	{
		r.x -= dx;
		r.w += dx * 2.f;
	}
	if( dy*2.f < -r.h )	{
		r.y += r.h / 2.f;
	}
	else	{
		r.y -= dy;
		r.h += dy * 2.f;
	}
}


void intersectRect( SDL_Rect &r, const SDL_Rect &r2 )
{
	if( r2.x > r.x )	{
		if( r2.x - r.x > r.w )	{
			setRectEmpty( r );
			return;
		}
		r.w -= r2.x - r.x;
		r.x = r2.x;
	}
	if( r2.y > r.y )	{
		if( r2.y - r.y > r.h )	{
			setRectEmpty( r );
			return;
		}
		r.h -= r2.y - r.y;
		r.y = r2.y;
	}
	register Sint16 v = r.x + r.w - ( r2.x + r2.w );
	if( v > 0 )	{
		if( v > r.w )	{
			setRectEmpty( r );
			return;
		}
		r.w -= v;
	}
	v = r.y + r.h - ( r2.y + r2.h );
	if( v > 0 )	{
		if( v > r.h )	{
			setRectEmpty( r );
			return;
		}
		r.h -= v;
	}
}

void intersectRect( FloatRect &r, const FloatRect &r2 )
{
	if( r2.x > r.x )	{
		if( r2.x - r.x > r.w )	{
			setRectEmpty( r );
			return;
		}
		r.w -= r2.x - r.x;
		r.x = r2.x;
	}
	if( r2.y > r.y )	{
		if( r2.y - r.y > r.h )	{
			setRectEmpty( r );
			return;
		}
		r.h -= r2.y - r.y;
		r.y = r2.y;
	}
	register float v = r.x + r.w - ( r2.x + r2.w );
	if( v > 0.f )	{
		if( v > r.w )	{
			setRectEmpty( r );
			return;
		}
		r.w -= v;
	}
	v = r.y + r.h - ( r2.y + r2.h );
	if( v > 0.f )	{
		if( v > r.h )	{
			setRectEmpty( r );
			return;
		}
		r.h -= v;
	}
}

