/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfilemusiclist.h"
#include "sdlfilemusic.h"


SdlFileMusicList::SdlFileMusicList( const char *fn, const char *name ): SdlMusicList(name? name : fn)
{
	ifstream file( fn, ios::in | ios::binary );
	if( !file.is_open() )	{
		failed();
		return;
	}

	chFN = new char[ strlen(fn) + 1 ];
	strcpy( chFN, fn );

	char chLine[252];
	file.getline( chLine, 252 );
	if( strncmp( chLine, "#EXTM3U", 7 ) == 0 )	{
		if( !loadM3U( &file ) )
			failed();
	}
	else if( strncmp( chLine, "[playlist]", 10 ) == 0 )	{
		if( !loadPLS( &file ) )
			failed();
	}
	else	{
		failed();
	}

}

SdlFileMusicList::~SdlFileMusicList()
{
	while( !lSongs.empty() )	{
		if( lSongs.back() != 0 )
			delete lSongs.back();
		if( lInfos.back() != 0 )
			delete lInfos.back();

		lSongs.pop_back();
		lInfos.pop_back();
	}
}

bool SdlFileMusicList::loadM3U( ifstream *file )
{
	char chLine[256], chTitle[256], chAuthor[256], chFName[256];
	int tSecs;
	while( file->good() )	{
		file->getline( chLine, 256 );
		int l = strlen( chLine );
		if( chLine[l-1] == '\r' )	{
			chLine[l-1] = '\0';
			--l;
		}
		if( l == 0 )
			continue;
		
		if( strncmp( chLine, "#EXTINF:", 8 ) == 0 )	{// read extended information
			if( strchr( chLine, ',' ) != 0 )	{
				char chTmp[32];
				int pos = strchr( chLine, ',' ) - chLine;
				strncpy( chTmp, chLine + 8, pos );
				tSecs = atoi( chTmp );

				int pos2;
				pos += 1;
				const char *pDash = strstr( chLine + pos, " - " );
				if( pDash != 0 )	{
					pos2 = pDash - chLine;
					strncpy( chAuthor, chLine + pos, pos2 - pos );
					chAuthor[pos2-pos] = '\0';

					strcpy( chTitle, pDash + 3 );
				}
				else	{
					strcpy( chAuthor, chLine + pos );
					chTitle[0] = 0;
				}
			}
		}
		else if( chLine[0] != '#' )	{// read song-filename and take it relative to list-filename
			char *chPathEnd = strrchr( chFN, '/' );
			if( chPathEnd == 0 )
				chPathEnd = chFN - 1;
			strncpy( chFName, chFN, chPathEnd - chFN + 1 );
			strcpy( &chFName[chPathEnd - chFN + 1], chLine );

			SdlMusic *m = new SdlFileMusic( chFName, (char*)-1 );
			if( !m->isFailed() )	{
				lSongs.push_back( m );
				lInfos.push_back( new SdlMusicInfo( chFName, chTitle, chAuthor, tSecs ) );
			}
			tSecs = -1;
			chAuthor[0] = 0;
			chTitle[0] = 0;
		}
	}
	return lSongs.size() != 0;
}

bool SdlFileMusicList::loadPLS( ifstream *file )
{
	char chLine[1024], chTitle[1024], chAuthor[1024], chFName[1024];

	while( !file->eof() )	{
		file->getline( chLine, 255 );
		if( strncmp( chLine, "NumberOfEntries=", 16 ) == 0 )	{
			int n = atoi( chLine + 16 );
			lSongs.reserve( n );
			lInfos.reserve( n );
			for( int i = 0; i < n; ++i )	{
				lSongs.push_back( 0 );
				lInfos.push_back( 0 );
			}
			break;
		}
//		else if( strncmp( chLine, "Version=", 8 ) == 0 )// evtl todo
//			continue;
	}

	file->seekg( 8, ios_base::beg );
	file->getline( chLine, 1024 );

	char chTmp[32];
	while( file->good() )	{
		file->getline( chLine, 1023 );
		int l = strlen( chLine );
		if( chLine[l-1] == '\r' )	{
			chLine[l-1] = '\0';
			--l;
		}
		if( l < 4 )
			continue;

		if( strncmp( chLine, "File", 4 ) == 0 )	{
			if( strchr( chLine, '=' ) == 0 )
				continue;
			int pos = strchr( chLine, '=' ) - chLine;
			strncpy( chTmp, chLine+4, pos - 4 );
			chTmp[pos-4] = '\0';
			unsigned int id = atoi( chTmp );
			if( lSongs.size() < id || id == 0 )	{
				continue;
//				lSongs.push_back( info );
//				id = lSongs.size();
			}
			
			if( chLine[ pos + 1 ] == '/' )	{
				// absolute file path
				strcpy( chFName, &chLine[ pos + 1 ] );
			}
			else	{
				// read song-filename and take it relative to list-filename
				char *chPathEnd = strrchr( chFN, '/' );
				if( chPathEnd == 0 )
					chPathEnd = chFN - 1;
				strncpy( chFName, chFN, chPathEnd - chFN + 1 );
				strcpy( &chFName[chPathEnd - chFN + 1], chLine+pos+1 );
			}
			SdlMusic *m = new SdlFileMusic( chFName, (char*)-1 );
			if( !m->isFailed() )	{
				if( lSongs.at( id-1 ) != 0 )
					delete lSongs.at( id-1 );
				lSongs.at( id-1 ) = m;
			}
			if( lInfos.at( id-1 ) == 0 )
				lInfos.at( id-1 ) = new SdlMusicInfo( chFName, 0, 0 );
			else lInfos.at( id-1 )->setFileName( chFName );
		}
		else if( strncmp( chLine, "Title", 4 ) == 0 )	{
			if( strchr( chLine, '=' ) == 0 )
				continue;
			int pos = strchr( chLine, '=' ) - chLine;
			strncpy( chTmp, chLine+5, pos - 5 );
			chTmp[pos-5] = '\0';
			unsigned int id = atoi( chTmp );
			if( lSongs.size() < id || id == 0 )	{
				continue;
//				lSongs.push_back( info );
//				id = lSongs.size();
			}

			pos += 1;
			const char *pDash = strstr( chLine + pos, " - " );
			if( pDash != 0 )	{
				int pos2 = pDash - chLine;
				strncpy( chAuthor, chLine + pos, pos2 - pos );
				chAuthor[pos2 - pos] = '\0';

				strcpy( chTitle, pDash + 3 );
			}
			else
				strcpy( chAuthor, chLine + pos );

			if( lInfos.at( id-1 ) == 0 )
				lInfos.at( id-1 ) = new SdlMusicInfo( 0, chTitle, chAuthor );
			else	{
				lInfos.at( id-1 )->setAuthor( chAuthor );
				lInfos.at( id-1 )->setTitle( chTitle );
			}
		}
		else if( strncmp( chLine, "Length", 6 ) == 0 )	{
			if( strchr( chLine, '=' ) == 0 )
				continue;
			int pos = strchr( chLine, '=' ) - chLine;
			strncpy( chTmp, chLine+6, pos - 6 );
			chTmp[pos-6] = '\0';
			unsigned int id = atoi( chTmp );
			if( lSongs.size() < id || id == 0 )	{
				continue;
//				lSongs.push_back( info );
//				id = lSongs.size();
			}
			if( lInfos.at( id-1 ) == 0 )
				lInfos.at( id-1 ) = new SdlMusicInfo( 0, 0, 0, atoi( chLine + pos + 1 ) );
			else	{
				lInfos.at( id-1 )->setLength( atoi( chLine + pos + 1 ) );
			}
		}
	}
	for( int i = 0 ; i < lInfos.size(); ++i )	{
		if( lInfos.at(i) == 0 )
			lInfos.at(i) = new SdlMusicInfo(0,0,0);
	}
	return lSongs.size() != 0;
}


