/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfilesurface.h"
#include "SDL/SDL_image.h"
#include "basicapp.h"
#include "definitions.h"


SdlFileSurface::SdlFileSurface( const char *fn, const char *name, ARGB_COLOR colorkey, const char *type, Uint32 flags )
	: SdlSurface( name != 0 ? name : fn )
{
	strFilename = fn;
	ulColorKey = colorkey;
	if( type )	{
		strncpy( chType, type, 3 );
		chType[3] = '\0';
	}
	else
		chType[0] = '\0';
	ulFlags = flags;
	
	surface = NULL;
	onRefresh();
}

SdlFileSurface::~SdlFileSurface()
{
}


void SdlFileSurface::onRefresh()
{
	if( surface != NULL )
		SDL_FreeSurface( surface );
	
	SDL_Surface *sysSurface;
	if( chType[0] == '\0' )
		sysSurface = IMG_Load( strFilename.c_str() );
	else
		sysSurface = IMG_LoadTyped_RW( SDL_RWFromFile( strFilename.c_str(), "rb" ), 1, chType );

	if( sysSurface == NULL )	{
		surface = NULL;
		BasicApp::the->logLevelLine( "ERROR: SdlFileSurface: Could not load image: " );
		BasicApp::the->logString( strFilename.c_str() );

		failed();
		return;
	}
	if( ulFlags & SDL_HWSURFACE )	{
		if( ulColorKey == CLR_INVALID )
			setSurface( SDL_DisplayFormatAlpha( sysSurface ) );
		else
			setSurface( SDL_DisplayFormat( sysSurface ) );
		SDL_FreeSurface( sysSurface );
	}
	else	{
		setSurface( sysSurface );
	}
	setColorKey( ulColorKey );
	
	if( surface == NULL )	{
		BasicApp::the->logLine( "Could not create BMP-surface: " );
		BasicApp::the->logString( strFilename.c_str() );

		failed();
		return;
	}
}

