/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfiletexture.h"
#include "SDL/SDL_image.h"
#include "basicapp.h"
#include "definitions.h"


SdlFileTexture::SdlFileTexture( const char *fn, const char *name, ARGB_COLOR colorkey, const char *type, bool mirror )
	: SdlTexture( name != 0 ? name : fn )
{
	strFilename = fn;
	ulColorKey = colorkey;
	if( type )	{
		strncpy( chType, type, 3 );
		chType[3] = '\0';
	}
	else
		chType[0] = '\0';
	bYMirror = mirror;
	
	onRefresh();
}

SdlFileTexture::~SdlFileTexture()
{
}


void SdlFileTexture::onRefresh()
{
	if( nId > 0 )
	{
		GLboolean bResident;
		glAreTexturesResident( 1, &nId, &bResident );
		if( bResident )
			return;
	}
	else
	{
		glGenTextures( 1, &nId );
	}
	glBindTexture( GL_TEXTURE_2D, nId );
	
	SDL_Surface *sysSurface;
	if( chType[0] == '\0' )
		sysSurface = IMG_Load( strFilename.c_str() );
	else
		sysSurface = IMG_LoadTyped_RW( SDL_RWFromFile( strFilename.c_str(), "rb" ), 1, chType );

	if( sysSurface == NULL )	{
		glDeleteTextures( 1, &nId );
		nId = 0;
		BasicApp::the->logLevelLine( "ERROR: SdlFileTexture: Could not load image: " );
		BasicApp::the->logString( strFilename.c_str() );

		failed();
		return;
	}
	
	Uint8 nBytesPerPixel = sysSurface->format->BytesPerPixel;
	if( ( ulColorKey == CLR_INVALID || nBytesPerPixel == 4 ) && !bYMirror )
	{
		glTexImage2D( GL_TEXTURE_2D, 0, nBytesPerPixel, sysSurface->w, sysSurface->h, 0,
					  nBytesPerPixel == 4 ? GL_BGRA : GL_BGR, GL_UNSIGNED_BYTE, sysSurface->pixels );
	}
	else
	{
		if( ulColorKey == CLR_INVALID )
			SDL_SetColorKey( sysSurface, SDL_SRCCOLORKEY, ulColorKey );
		else
			SDL_SetColorKey( sysSurface, SDL_SRCCOLORKEY, SDL_MapRGB(sysSurface->format, ulColorKey>>16, ulColorKey>>8, ulColorKey) );
		GLenum eFormat;
		unsigned char *pPixels = pixelsFromSurface4ub( sysSurface, eFormat, bYMirror );
		if( pPixels )
		{
			glTexImage2D( GL_TEXTURE_2D, 0, 4, sysSurface->w, sysSurface->h, 0,
						eFormat, GL_UNSIGNED_BYTE, pPixels );
			
			delete[] pPixels;
		}
		else
		{
			glDeleteTextures( 1, &nId );
			nId = 0;
			BasicApp::the->logLevelLine( "ERROR: SdlFileTexture: Could not copy image: " );
			BasicApp::the->logString( strFilename.c_str() );
			
			failed();
			SDL_FreeSurface( sysSurface );
			return;
		}
	}
	SDL_FreeSurface( sysSurface );
	
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	
	nWidth = sysSurface->w;
	nHeight = sysSurface->h;
}

