/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlfontsurfaces.h"
#include "SDL/SDL_ttf.h"
#include "basicapp.h"
#include "functions.h"

unsigned int SdlFontSurfaces::nRefCount = 0;

SdlFontSurfaces::SdlFontSurfaces( const char *ttf_file, const char *letters, int ptsize, SDL_Color c, const char *name/*, SdlSurface *bmp*/ ): SdlSurfaceContainer( name ? name : ttf_file )
{
	chFileName = new char[ strlen( ttf_file ) + 1 ];
	strcpy( chFileName, ttf_file );

	chLetters = new char[ strlen( letters ) + 1 ];
	strcpy( chLetters, letters );

	nPtSize = ptsize;
	color = c;
	//bgBmp = bmp;

	if( !TTF_WasInit() )	{
		if( TTF_Init() == -1 )	{
			failed();
			BasicApp::the->logLevelLine("ERROR: SdlFontSurfaces: cannot initialize TTF engine: ");
			BasicApp::the->logString( TTF_GetError() );
			return;
		}
		nRefCount = 1;
	}
	else
		++nRefCount;

	onRefresh();
}

SdlFontSurfaces::~SdlFontSurfaces()
{
	for( unsigned int i = 0; i < lSurfaces.size(); ++i )	{
		delete lSurfaces[i];
	}
	delete[] chFileName;
	if( --nRefCount == 0 )
		TTF_Quit();
}

void SdlFontSurfaces::onRefresh()
{
//	static const SDL_Color transparentColor = {255,255,255,1};

	TTF_Font *font = TTF_OpenFont( chFileName, nPtSize );
	if( font == 0 )	{
		BasicApp::the->logLevelLine( "ERROR: Cannot load ttf-fontfile." );
		failed();
		return;
	}

	char t[2];
	t[1] = '\0';

	int w, h,
		max_width = 0;
	char *l;
	for( l = chLetters; *l != '\0'; ++l )	{
		t[0] = *l;
		if(	TTF_SizeText( font, t, &w, &h ) != -1 )
			if( w > max_width )
				max_width = w;
	}

	SDL_Surface *surf;
	SdlSurface *graphic;
	h = TTF_FontHeight( font );
	SDL_PixelFormat *f = SdlManager::the->getDisplay()->surf()->format;
	SDL_Rect rdst;
	rdst.h = h;
	rdst.y = 0;
	if ( h == 0 || w == 0 )	{
		BasicApp::the->logLevelLine( "ERROR: Invalid ttf-font." );
		failed();
		return;
	}

	while( lSurfaces.size() )	{
		delete lSurfaces.back();
		lSurfaces.pop_back();
	}

	lSurfaces.reserve( strlen(chLetters) );
	for( l = chLetters; *l != '\0'; ++l )	{
		graphic = new SdlSurface( SDL_CreateRGBSurface( SDL_HWSURFACE, max_width, h, f->BitsPerPixel, f->Rmask, f->Gmask, f->Bmask, f->Amask ), (char*)-1 );
		
		SDL_FillRect( graphic->surf(), 0, RGB(0,0,0) );
		if( *l != ' ' )	{
			t[0] = *l;
	/*		if( bgBmp )
				surf = TTF_RenderText_Solid( font, t, transparentColor );
			else*/
				surf = TTF_RenderText_Solid( font, t, color );
			if( !surf )	{
				BasicApp::the->logLevelLine( "ERROR: Cannot initialize font: " );
				BasicApp::the->logString( TTF_GetError() );
				BasicApp::the->logString( ", at letter: " );
				BasicApp::the->logString( t );
				failed();
				break;
			}
			rdst.w = surf->w;
			rdst.x = (max_width - surf->w) / 2;
			/*
			if( bgBmp )	{ --> ERROR here: no transparency, why???
				SDL_SetColorKey( surf, SDL_SRCCOLORKEY | SDL_RLEACCEL, RGB(transparentColor.r,transparentColor.g,transparentColor.b) );
				SDL_BlitSurface( bgBmp->surf(), &rdst, graphic->surf(), &rdst );
			}*/
			SDL_BlitSurface( surf, 0, graphic->surf(), &rdst );
			SDL_FreeSurface( surf );
		}
		graphic->setColorKey( RGB(0,0,0) );
		
		lSurfaces.push_back( graphic );
	}

	TTF_CloseFont( font );
}

