/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2007 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlimage.h"
#include <string.h>

#include "definitions.h"
#include "sdlmanager.h"
#include "basicapp.h"

SdlImage::SdlImage( const char *name )
{
	bFailed = false;
	initName( name );
	nWidth = nHeight = 0;
}


SdlImage::~SdlImage()
{
	if( chName )
	    delete[] chName;

	if( bFailed )
		SdlManager::the->lFailedImages.remove( this );
	else
		SdlManager::the->lImages.remove( this );
}

void SdlImage::failed()
{
	SdlManager::the->lImages.remove( this );
	SdlManager::the->lFailedImages.push_back( this );

	bFailed = true;
}

void SdlImage::initName( const char *name )
{
	if( ( name == 0 )||( name == (char *)-1 ) )	{
		chName = 0;
		return;
	}
	if( SdlManager::the->getImageByName( name ) != NULL )	{
		BasicApp::the->logLine( "WARNING: Name of image already availible: " );
		BasicApp::the->logString( name );
	}
	int n = strlen( name );
	chName = new char[n + 1];
	memcpy( chName, name, n );
	chName[n] = 0;

	SdlManager::the->lImages.push_back( this );
}

void SdlImage::rename( const char *newname )
{
	if( chName != 0 )
		delete chName;
	else
		if( newname != 0 )
			if( bFailed )
				SdlManager::the->lFailedImages.push_back( this );
			else
				SdlManager::the->lImages.push_back( this );

	if( newname == 0 )	{
		if( bFailed )
			SdlManager::the->lFailedImages.remove( this );
		else
			SdlManager::the->lImages.remove( this );
		chName = 0;
	}
	else	{
		int n = strlen( newname );
		chName = new char[n + 1];
		memcpy( chName, newname, n );
		chName[n] = 0;
	}
}


// -- SdlImageContainer --

SdlImageContainer::SdlImageContainer( const char *name )
{
	bFailed = false;

	chName = new char[strlen( name ) + 1];
	strcpy( chName, name );

	SdlManager::the->lImageContainers.push_back( this );
}

SdlImageContainer::~SdlImageContainer()
{
	delete[] chName;

	if( bFailed )
		SdlManager::the->lFailedImageContainers.remove( this );
	else
		SdlManager::the->lImageContainers.remove( this );
}

void SdlImageContainer::failed()
{
	SdlManager::the->lImageContainers.remove( this );
	SdlManager::the->lFailedImageContainers.push_back( this );
	bFailed = true;
}

