/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "sdlmusiclist.h"
#include "sdlmanager.h"
#include "sdlfilemusic.h"


SdlMusicInfo::SdlMusicInfo( const char *fn, const char *title, const char *author, int secs )
{
	if( fn )	{
		chName = new char[ strlen( fn ) + 1 ];
		strcpy( chName, fn );
	}
	else chName = 0;
	if( title )	{
		chTitle = new char[ strlen( title ) + 1 ];
		strcpy( chTitle, title );
	}
	else chTitle = 0;
	if( author )	{
		chAuthor = new char[ strlen( author ) + 1 ];
		strcpy( chAuthor, author );
	}
	else chAuthor = 0;

	nLength = secs;// -1 = unknown
	nPlayed = 0;
}

SdlMusicInfo::~SdlMusicInfo()
{
	if( chName != 0 )
		delete[] chName;
	if( chTitle != 0 )
		delete[] chTitle;
	if( chAuthor != 0 )
		delete[] chAuthor;
}

void SdlMusicInfo::setFileName( const char *n )
{
	if( chName )
		delete[] chName;
	chName = new char[ strlen( n ) + 1 ];
	strcpy( chName, n );
}

void SdlMusicInfo::setTitle( const char *t )
{
	if( chTitle )
		delete[] chTitle;
	chTitle = new char[ strlen( t ) + 1 ];
	strcpy( chTitle, t );
}

void SdlMusicInfo::setAuthor( const char *a )
{
	if( chAuthor )
		delete[] chAuthor;
	chAuthor = new char[ strlen( a ) + 1 ];
	strcpy( chAuthor, a );
}





SdlMusicList::SdlMusicList( const char *name )
{
	bFailed = false;
	bStopped = true;

	chName = new char[strlen( name ) + 1];
	strcpy( chName, name );
	nVolume = 128;

	SdlManager::the->lMusicLists.push_back( this );
}

SdlMusicList::~SdlMusicList()
{
	delete[] chName;

	SdlManager::the->lMusicLists.remove( this );
}

void SdlMusicList::failed()
{
	bFailed = true;
}


SdlMusic *SdlMusicList::getSong( unsigned int i )
{
	if( i >= lSongs.size() )
		return 0;
	return lSongs[i];
}

SdlMusicInfo *SdlMusicList::getSongInfo( unsigned int i )
{
	if( i >= lInfos.size() )
		return 0;
	return lInfos[i];
}

bool SdlMusicList::play( PlayTypes t )
{
	if( numSongs() == 0 )
		return false;
	ePlayType = t;
	switch( ePlayType )	{
	case ptForward:
		nCurrentSong = -1;
		break;
	case ptBackward:
		nCurrentSong = 0;
		break;
	}
	for( int i = 0 ; i < numSongs(); ++i )
		lInfos.at(i)->reset();

	return playNext();
}

void SdlMusicList::stop()
{
	bStopped = true;
	if( musPlaying != 0 )	{
		musPlaying->stop();
		musPlaying = 0;
	}
}

void SdlMusicList::setVolume( int v )
{
	if( v >= 0 )
		nVolume = v;
	if( !bStopped )
		musPlaying->setVolume( nVolume );
}

bool SdlMusicList::playNext()
{
	bStopped = false;
	if( numSongs() < 2 )
		nCurrentSong = 0;
	else
	 switch( ePlayType )	{
	  case ptForward:
		++nCurrentSong;
		if( nCurrentSong == numSongs() )	{
			if( cb )
				(*cb)( this, cbData );
			nCurrentSong = 0;
		}
		break;

	  case ptBackward:
		--nCurrentSong;
		if( -nCurrentSong == numSongs() + 1 )	{
			if( cb )
				(*cb)( this, cbData );
			nCurrentSong = -1;
		}
		break;

	  case ptShuffled:	{
		int bDone = true;
		int min = lInfos.at(0)->numPlayed();// Annahme: song 0 weniger gespielt
		int nUnplayed = 1;
		int i;
		for( i = 1; i < numSongs(); ++i )
			if( lInfos.at(i)->numPlayed() != min )	{
				bDone = false;
				if( lInfos.at(i)->numPlayed() < min )	{
					min = lInfos.at(i)->numPlayed();
					nUnplayed = 1;
				}
			}
			else
				++nUnplayed;

		if( bDone )	{
			if( cb )
				(*cb)( this, cbData );

			nCurrentSong = rand() % numSongs();
		}
		else	{
			if( nUnplayed == 1 )
				nCurrentSong = 1;
			else
				nCurrentSong = rand() % nUnplayed + 1;
			for( i = 0; i < nCurrentSong; ++i )
				if( lInfos.at(i)->numPlayed() != min )
					++nCurrentSong;
		}

		break;
	  }

	  case ptRandom:
		nCurrentSong = rand() % numSongs();
		break;
	 }

	if( bStopped )// may be stopped during this function in a callback
		return false;

	if( lSongs.at(nCurrentSong)->play() == 0 )	{
		musPlaying = lSongs.at(nCurrentSong);
		musPlaying->setVolume( nVolume );
		musPlaying->setCallBack( onFinished, this );

		lInfos.at(nCurrentSong)->played();
	}
	else musPlaying = 0;

	return musPlaying != 0;
}
