/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "frametimer.h"

#define INFINITE	0xFFFFFFFF

FrameTimer *FrameTimer::the = NULL;
FrameTimer *FrameTimer::getThe()
{
	return the;
}

FrameTimer::FrameTimer()
{
	if( the ) return;
	the = this;
}

FrameTimer::~FrameTimer()
{
	while( cbsInactive.size() )	{
		delete cbsInactive.back();
		cbsInactive.pop_back();
	}
	while( cbs.size() )	{
		delete cbs.back();
		cbs.pop_back();
	}
}

FrameTimer::CbInfo * FrameTimer::addCallback( Callback c, BasicObject * d, Uint32 iv )
{
	CbInfo * cb = new CbInfo;
	cb->init( c, d, iv, iv == INFINITE ? INFINITE : SDL_GetTicks() + iv );
	if( iv == INFINITE )
		cbsInactive.push_back( cb );
	else
		cbs.push_back( cb );
	return cb;
}

void FrameTimer::removeCallback( CbInfo * pCb )
{
	cbs.remove( pCb );
	cbsInactive.remove( pCb );
}

void FrameTimer::doTimer()
{
	Uint32 t = SDL_GetTicks();
	for( CbInfoIter i = cbs.begin(); i != cbs.end(); i++ )
FOR_ENTRY:
		if(t >= (*i)->tNext)
		{
			(*i)->tNext += (*i)->tInterval;
			if((*i)->cb)
				(*(*i)->cb)(this,(*i)->pData,NULL);
		}
		else if( (*i)->tInterval == INFINITE )	{
			cbsInactive.push_back( *i );
			CbInfoIter i2 = i;
			++i;
			cbs.erase( i2 );
			if( i == cbs.end() )
				break;
			else goto FOR_ENTRY;
		}
}

bool FrameTimer::setInterval( CbInfo * pinfo, Uint32 iv )
{
	if( pinfo == NULL )
		return false;
	if( iv == pinfo->tInterval )
		return true;
	if( iv == INFINITE )	{
		pinfo->tNext = INFINITE;
	}
	else	{
		if( pinfo->tInterval == INFINITE )	{
			for( CbInfoIter i = cbsInactive.begin(); i != cbsInactive.end(); ++i )	{
				if( *i == pinfo )	{
					cbsInactive.erase( i );
					cbs.push_back( pinfo );
					break;
				}
			}
		}
		pinfo->tNext = iv + SDL_GetTicks();
	}
	pinfo->tInterval = iv;
	return true;
}
