/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "thememanager.h"
#include "uitheme.h"
#include "uimanager.h"


ThemeManager *ThemeManager::the = 0;
ThemeManager *ThemeManager::getThe()
{
	return the;
}

ThemeManager::ThemeManager( LoadScheme ls )
{
	the = this;

	thActive = 0;

	setLoadScheme( ls );
}

ThemeManager::~ThemeManager()
{
	the = 0;
}

void ThemeManager::setLoadScheme( LoadScheme ls )
{
	lsThemes = ls;

	switch( ls )
	{
	case lsOnAppStart:
		{
			for( ThemeIter i = lThemes.begin(); i != lThemes.end(); ++i )
				if( !(*i)->isLoaded() )
					(*i)->onLoad(false);
			break;
		}
	default:
		break;
	};
}

bool ThemeManager::setActiveTheme( UiTheme *t, bool hidden )
{
	if( thActive == t )
		return true;
	if( t == 0 )
		return false;

	if( !t->isLoaded() )
		if( t->onLoad(hidden) != 0)
			return false;

	if( thActive )
		thActive->onDeactivate();
	thActive = t;

	bool r = t->onActivate() == 0;
	UiManager::the->themeChanged( thActive );
	return r;
}

UiTheme * ThemeManager::getThemeByName( const char *name )
{
	for( ThemeIter i = lThemes.begin(); i != lThemes.end(); ++i )
		if( strcmp( (*i)->chName, name ) == 0 )
			return *i;

	return 0;
}

