/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "uikeycombination.h"
#include "uimanager.h"
#include "basicapp.h"
#include "functions.h"

using namespace std;

UiKeyCombination *UiKeyCombination::the = NULL;

UiKeyCombination *UiKeyCombination::getThe()
{
	return the;
}


UiKeyCombination::UiKeyCombination()
{
	the = this;
}

UiKeyCombination::~UiKeyCombination()
{
	the = NULL;
	removeAll();
}

void UiKeyCombination::init( TiXmlElement *el )
{
	BasicApp::the->logBegin("Initializing UiKeyCombination");
	TiXmlElement *combi;
	for( combi = el->FirstChildElement( "Combi" ); combi; combi = combi->NextSiblingElement( "Combi" ) )	{
		SDLKey k;
		SDLMod m;
		BasicObject::FunctionData p, r;
		p.func = NULL;
		r.func = NULL;
		const char *ch = combi->Attribute("Key");
		if(ch)
			k = strToKey(ch);
		else continue;
		ch = combi->Attribute("Mod");
		if(ch)
			m = strToMod(ch);
		else continue;
		ch = combi->Attribute("OnPress");
		if(ch)	{
			p = BasicObject::getFunction(ch);
		}
		ch = combi->Attribute("OnRelease");
		if(ch)	{
			r = BasicObject::getFunction(ch);
		}
		addCombination( k, m, p.obj, p.func, r.obj, r.func );
	}
	BasicApp::the->logEnd("done.");
}

void UiKeyCombination::addCombination( SDLKey k, SDLMod m, BasicObject *aim_p, BasicObject::Callback p, BasicObject *aim_r, BasicObject::Callback r )
{
	for( KeyCombIter i = lKeyCombs.begin(); i != lKeyCombs.end(); ++i )	{
		if( ((*i)->mod == m) && ((*i)->key == k) )	{
			KeyComb *kc = *i;
			kc->pressed.func = p;// rein theoretisch nicht thread-safe; IMO praktisch schon
			kc->pressed.obj = aim_p;
			kc->released.func = r;
			kc->released.obj = aim_r;
			return;
		}
	}
	KeyComb *kc;
	kc = new KeyComb;
	kc->mod = m;
	kc->key = k;
	kc->pressed.func = p;// rein theoretisch nicht thread-safe; IMO praktisch schon
	kc->pressed.obj = aim_p;
	kc->released.func = r;
	kc->released.obj = aim_r;
	lKeyCombs.push_back( kc );
}

bool UiKeyCombination::removeCombination( SDLKey k, SDLMod m )
{
	for( KeyCombIter i = lKeyCombs.begin(); i != lKeyCombs.end(); ++i )	{
		if( (*i)->mod == m && (*i)->key == k )	{
			delete *i;
			lKeyCombs.erase( i );
			return true;
		}
	}
	return false;
}

void UiKeyCombination::removeAll()
{
	while(!lKeyCombs.empty())	{
		delete lKeyCombs.front();
		lKeyCombs.pop_front();
	}
}

bool UiKeyCombination::keyPressed( SDLKey k, SDLMod m )
{
	for( KeyCombIter i = lKeyCombs.begin(); i != lKeyCombs.end(); ++i )	{
		if( (*i)->mod == m && (*i)->key == k && (*i)->pressed.func )	{
			(*(*i)->pressed.func)( this, (*i)->pressed.obj, NULL );
			return true;
		}
	}
	return false;
}

bool UiKeyCombination::keyReleased( SDLKey k, SDLMod m )
{
	for( KeyCombIter i = lKeyCombs.begin(); i != lKeyCombs.end(); ++i )	{
		if( (*i)->mod == m && (*i)->key == k && (*i)->released.func )	{
			(*(*i)->released.func)( this, (*i)->released.obj, NULL );
			return true;
		}
	}
	return false;
}

