/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 1999, 2000, 2005 by Tobias Nadler und Georg Schnattinger      */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "uiobject.h"
#include "uimanager.h"
#include "thememanager.h"


UiObject::UiObject( BasicObject *parent, const char *name ): BasicObject(name, parent)
{
}

UiObject::UiObject( TiXmlElement *el, BasicObject *parent ): BasicObject(el, parent)
{
}

UiObject::~UiObject()
{
}

void UiObject::onDestroy()
{
	if( !UiManager::the )
		return;
	if( UiManager::the->getFocus( ftKeyboard ) == this )
		UiManager::the->setFocus( ftKeyboard, NULL );
	if( UiManager::the->getFocus( ftMouse ) == this )
		UiManager::the->setFocus( ftMouse, NULL );
	if( UiManager::the->getFocus( ftDisplay ) == this )
		UiManager::the->setFocus( ftDisplay, NULL );
	
	BasicObject::onDestroy();
}

void UiObject::toTop()
{
	if( getParent() )	{
		getParent()->children.erase(parentitem);
		parentitem = getParent()->children.insert(getParent()->children.begin(), this);
	}
}

void UiObject::toBottom()
{
	if( getParent() )	{
		getParent()->children.erase(parentitem);
		parentitem = getParent()->children.insert(getParent()->children.end(), this);
	}
}

void UiObject::oneStepToTop()
{
	if( getParent() )	{
		child_iter i = getParent()->children.erase(parentitem);
		if(i != getParent()->children.begin())
			i--;
		parentitem = getParent()->children.insert(i,this);
	}
}

void UiObject::oneStepToBottom()
{
	if( getParent() )	{
		child_iter i = getParent()->children.erase(parentitem);
		if(i != getParent()->children.end())
			i++;
		parentitem = getParent()->children.insert(i,this);
	}
}
/*
Uint32 UiObject::onFrame()
{
	Uint32 r;
	for( child_reverse_iter i = children.rbegin(); i != children.rend(); ++i )	{
		if( (*i)->isUiObject() && ( r = ((UiObject*)(*i))->onFrame() ) != 0 )
			return r;
	}
	return 0;
}*/

Uint32 UiObject::onThemeChanged( UiTheme *t )
{
	Uint32 r;
	for( child_iter i = children.begin(); i != children.end(); ++i )	{
		if( (*i)->isUiObject() && ( r = ((UiObject*)(*i))->onThemeChanged( t ) ) != 0 )
			return r;
	}
	return 0;
}


const char *UiObject::tr( const char *text )
{
	if( !ThemeManager::the )
		return text;
	UiTheme *t = ThemeManager::the->getActiveTheme();
	if( t == 0 )
		return text;

	return t->translate( text );
}

