/********************************************************************************/
/*                                                                              */
/*  Netrinjo-Engine - a library for game-development                            */
/*  Copyright (C) 2005 by Tobias Nadler                                         */
/*                                                                              */
/*  This library is free software; you can redistribute it and/or               */
/*  modify it under the terms of the GNU Lesser General Public                  */
/*  License as published by the Free Software Foundation; either                */
/*  version 2.1 of the License, or (at your option) any later version.          */
/*                                                                              */
/*  This library is distributed in the hope that it will be useful,             */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of              */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           */
/*  Lesser General Public License for more details.                             */
/*                                                                              */
/*  You should have received a copy of the GNU Lesser General Public            */
/*  License along with this library; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   */
/*                                                                              */
/********************************************************************************/

#include "uitranslator.h"
#include <fstream>
#include "basicapp.h"

using namespace std;

UiTranslator::UiTranslator( const char *language )
{
	if( language )	{
		chLanguage = new char[ strlen(language) + 1 ];
		strcpy( chLanguage, language );
	}
	else chLanguage = 0;
}

UiTranslator::~UiTranslator()
{
	if( chLanguage )
		delete[] chLanguage;
}

void UiTranslator::setLanguage( const char * lang )
{
	if( chLanguage )
		delete[] chLanguage;
	if( lang )	{
		chLanguage = new char[ strlen(lang) + 1 ];
		strcpy( chLanguage, lang );
	}
	else chLanguage = 0;
}




UiFileTranslator::UiFileTranslator( const char *fn ): UiTranslator( 0 )
{
	if( fn )
		load( fn );
}

UiFileTranslator::~UiFileTranslator()
{
	for( int i = 0; i < 27; ++i )	{
		for( list<EXPRESSION>::iterator it = lExpressions[i].begin(); it != lExpressions[i].end(); ++it )
		{
			if( (*it).chCoded != 0 )
				delete[] (*it).chCoded;
			if( (*it).chTranslated != 0 )
				delete[] (*it).chTranslated;
		}
		lExpressions[i].clear();
	}
}

bool UiFileTranslator::load( const char *fn )
{
	BasicApp::the->logBegin( "Loading translation-file..." );
	ifstream file ( fn, ios::in | ios::binary );
	if( !file.is_open() )	{
		BasicApp::the->logEnd( "ERROR: file cannot be opened!" );
		return false;
	}

	char chLine[512];
	while( !file.eof() )	{
		file.getline( chLine, 512 );
		int l = strlen( chLine );
		if( chLine[l-1] == '\r' )	{
			chLine[l-1] = '\0';
			--l;
		}
		if( l < 3 )
			continue;
		if( chLine[0] == '#' )	{
			if( strncmp( &chLine[1], "LANG=", 5 ) == 0 )
				setLanguage( &chLine[6] );
			continue;
		}
		char *chCoded = chLine;
		while( chCoded[0] == ' ' || chCoded[0] == '\t' )	{
			++chCoded;
			--l;
		}
		if( l < 3 )
			continue;
		char *chTranslated = chCoded;
		while( 1 )	{
			chTranslated = strstr( chTranslated, "=>" );
			if( chTranslated != 0 )
				if( *(chTranslated-1) == '\\' )
					++chTranslated;
				else
					break;
		}
		chTranslated[0] = '\0';
		// clear spaces at end of coded-param
		int n;
		for( n = 1; chCoded < chTranslated-n; ++n )
			switch( *(chTranslated-n) )	{
			case ' ':
			case '\t':
				*(chTranslated-n) = '\0';
				break;
			default:
				n = 256;// break for
			}
		// setting pointer after the '=>'
		chTranslated += 2;

		// clear spaces at beginning of translated-param
		while( chTranslated[0] == ' ' || chTranslated[0] == '\t' )	{
			++chTranslated;
		}
		// and at end
		for( n = strlen(chTranslated); n > 0; --n )
			switch( *(chTranslated + n) )	{
			case ' ':
			case '\t':
				*(chTranslated + n) = '\0';
				break;
			default:
				n = 0;// break for
			}

		EXPRESSION ex;
		ex.chCoded = new char[ strlen(chCoded) + 1 ];
		ex.chTranslated = new char[ strlen(chTranslated) + 1 ];
		strcpy( ex.chCoded, chCoded );
		strcpy( ex.chTranslated, chTranslated );

		// make '\=>' to '=>', if some exist
		chCoded = ex.chCoded;
		while( (chCoded = strstr( chCoded, "\\=>" )) != 0 )	{
			strcpy( chCoded, chCoded+1 );
		}
		chTranslated = ex.chTranslated;
		while( (chTranslated = strstr( chTranslated, "\\=>" )) != 0 )	{
			strcpy( chTranslated, chTranslated+1 );
		}

		int i = toupper( ex.chCoded[0] ) - 'A';
		if( i < 0 || i > 26 )
			i = 26;
		lExpressions[i].push_back( ex );
	}

	file.close();
	BasicApp::the->logEnd( "done." );
	return true;
}

const char *UiFileTranslator::translate( const char *expression )
{
	int i = toupper( expression[0] ) - 'A';
	if( i < 0 || i > 26 )
		i = 26;
	for( list<EXPRESSION>::iterator it = lExpressions[i].begin(); it != lExpressions[i].end(); ++it )
		if( strcmp( (*it).chCoded, expression ) == 0 )
			return (*it).chTranslated;

	return expression;
}

